@@ -16,8 +16,8 @@ extern "C" {
  * objects created by functions in secp256k1.h, i.e., they can be passed to any
  * API function that expects a context object (see secp256k1.h for details). The
  * only exception is that context objects created by functions in this module
- * must be destroyed using rustsecp256k1zkp_v0_5_0_context_preallocated_destroy (in this
- * module) instead of rustsecp256k1zkp_v0_5_0_context_destroy (in secp256k1.h).
+ * must be destroyed using rustsecp256k1zkp_v0_6_0_context_preallocated_destroy (in this
+ * module) instead of rustsecp256k1zkp_v0_6_0_context_destroy (in secp256k1.h).
  *
  * It is guaranteed that functions in this module will not call malloc or its
  * friends realloc, calloc, and free.
@@ -27,24 +27,24 @@ extern "C" {
  *  caller-provided memory.
  *
  *  The purpose of this function is to determine how much memory must be provided
- *  to rustsecp256k1zkp_v0_5_0_context_preallocated_create.
+ *  to rustsecp256k1zkp_v0_6_0_context_preallocated_create.
  *
  *  Returns: the required size of the caller-provided memory block
  *  In:      flags:    which parts of the context to initialize.
  */
-SECP256K1_API size_t rustsecp256k1zkp_v0_5_0_context_preallocated_size(
+SECP256K1_API size_t rustsecp256k1zkp_v0_6_0_context_preallocated_size(
     unsigned int flags
 ) SECP256K1_WARN_UNUSED_RESULT;
 
 /** Create a secp256k1 context object in caller-provided memory.
  *
  *  The caller must provide a pointer to a rewritable contiguous block of memory
- *  of size at least rustsecp256k1zkp_v0_5_0_context_preallocated_size(flags) bytes, suitably
+ *  of size at least rustsecp256k1zkp_v0_6_0_context_preallocated_size(flags) bytes, suitably
  *  aligned to hold an object of any type.
  *
  *  The block of memory is exclusively owned by the created context object during
  *  the lifetime of this context object, which begins with the call to this
- *  function and ends when a call to rustsecp256k1zkp_v0_5_0_context_preallocated_destroy
+ *  function and ends when a call to rustsecp256k1zkp_v0_6_0_context_preallocated_destroy
  *  (which destroys the context object again) returns. During the lifetime of the
  *  context object, the caller is obligated not to access this block of memory,
  *  i.e., the caller may not read or write the memory, e.g., by copying the memory
@@ -54,14 +54,14 @@ SECP256K1_API size_t rustsecp256k1zkp_v0_5_0_context_preallocated_size(
  *
  *  Returns: a newly created context object.
  *  In:      prealloc: a pointer to a rewritable contiguous block of memory of
- *                     size at least rustsecp256k1zkp_v0_5_0_context_preallocated_size(flags)
- *                     bytes, as detailed above (cannot be NULL)
+ *                     size at least rustsecp256k1zkp_v0_6_0_context_preallocated_size(flags)
+ *                     bytes, as detailed above.
  *           flags:    which parts of the context to initialize.
  *
- *  See also rustsecp256k1zkp_v0_5_0_context_randomize (in secp256k1.h)
- *  and rustsecp256k1zkp_v0_5_0_context_preallocated_destroy.
+ *  See also rustsecp256k1zkp_v0_6_0_context_randomize (in secp256k1.h)
+ *  and rustsecp256k1zkp_v0_6_0_context_preallocated_destroy.
  */
-SECP256K1_API rustsecp256k1zkp_v0_5_0_context* rustsecp256k1zkp_v0_5_0_context_preallocated_create(
+SECP256K1_API rustsecp256k1zkp_v0_6_0_context* rustsecp256k1zkp_v0_6_0_context_preallocated_create(
     void* prealloc,
     unsigned int flags
 ) SECP256K1_ARG_NONNULL(1) SECP256K1_WARN_UNUSED_RESULT;
@@ -70,30 +70,30 @@ SECP256K1_API rustsecp256k1zkp_v0_5_0_context* rustsecp256k1zkp_v0_5_0_context_p
  *  caller-provided memory.
  *
  *  Returns: the required size of the caller-provided memory block.
- *  In:      ctx: an existing context to copy (cannot be NULL)
+ *  In:      ctx: an existing context to copy.
  */
-SECP256K1_API size_t rustsecp256k1zkp_v0_5_0_context_preallocated_clone_size(
-    const rustsecp256k1zkp_v0_5_0_context* ctx
+SECP256K1_API size_t rustsecp256k1zkp_v0_6_0_context_preallocated_clone_size(
+    const rustsecp256k1zkp_v0_6_0_context* ctx
 ) SECP256K1_ARG_NONNULL(1) SECP256K1_WARN_UNUSED_RESULT;
 
 /** Copy a secp256k1 context object into caller-provided memory.
  *
  *  The caller must provide a pointer to a rewritable contiguous block of memory
- *  of size at least rustsecp256k1zkp_v0_5_0_context_preallocated_size(flags) bytes, suitably
+ *  of size at least rustsecp256k1zkp_v0_6_0_context_preallocated_size(flags) bytes, suitably
  *  aligned to hold an object of any type.
  *
  *  The block of memory is exclusively owned by the created context object during
  *  the lifetime of this context object, see the description of
- *  rustsecp256k1zkp_v0_5_0_context_preallocated_create for details.
+ *  rustsecp256k1zkp_v0_6_0_context_preallocated_create for details.
  *
  *  Returns: a newly created context object.
- *  Args:    ctx:      an existing context to copy (cannot be NULL)
+ *  Args:    ctx:      an existing context to copy.
  *  In:      prealloc: a pointer to a rewritable contiguous block of memory of
- *                     size at least rustsecp256k1zkp_v0_5_0_context_preallocated_size(flags)
- *                     bytes, as detailed above (cannot be NULL)
+ *                     size at least rustsecp256k1zkp_v0_6_0_context_preallocated_size(flags)
+ *                     bytes, as detailed above.
  */
-SECP256K1_API rustsecp256k1zkp_v0_5_0_context* rustsecp256k1zkp_v0_5_0_context_preallocated_clone(
-    const rustsecp256k1zkp_v0_5_0_context* ctx,
+SECP256K1_API rustsecp256k1zkp_v0_6_0_context* rustsecp256k1zkp_v0_6_0_context_preallocated_clone(
+    const rustsecp256k1zkp_v0_6_0_context* ctx,
     void* prealloc
 ) SECP256K1_ARG_NONNULL(1) SECP256K1_ARG_NONNULL(2) SECP256K1_WARN_UNUSED_RESULT;
 
@@ -103,23 +103,23 @@ SECP256K1_API rustsecp256k1zkp_v0_5_0_context* rustsecp256k1zkp_v0_5_0_context_p
  *  The context pointer may not be used afterwards.
  *
  *  The context to destroy must have been created using
- *  rustsecp256k1zkp_v0_5_0_context_preallocated_create or rustsecp256k1zkp_v0_5_0_context_preallocated_clone.
- *  If the context has instead been created using rustsecp256k1zkp_v0_5_0_context_create or
- *  rustsecp256k1zkp_v0_5_0_context_clone, the behaviour is undefined. In that case,
- *  rustsecp256k1zkp_v0_5_0_context_destroy must be used instead.
+ *  rustsecp256k1zkp_v0_6_0_context_preallocated_create or rustsecp256k1zkp_v0_6_0_context_preallocated_clone.
+ *  If the context has instead been created using rustsecp256k1zkp_v0_6_0_context_create or
+ *  rustsecp256k1zkp_v0_6_0_context_clone, the behaviour is undefined. In that case,
+ *  rustsecp256k1zkp_v0_6_0_context_destroy must be used instead.
  *
  *  If required, it is the responsibility of the caller to deallocate the block
  *  of memory properly after this function returns, e.g., by calling free on the
- *  preallocated pointer given to rustsecp256k1zkp_v0_5_0_context_preallocated_create or
- *  rustsecp256k1zkp_v0_5_0_context_preallocated_clone.
+ *  preallocated pointer given to rustsecp256k1zkp_v0_6_0_context_preallocated_create or
+ *  rustsecp256k1zkp_v0_6_0_context_preallocated_clone.
  *
  *  Args:   ctx: an existing context to destroy, constructed using
- *               rustsecp256k1zkp_v0_5_0_context_preallocated_create or
- *               rustsecp256k1zkp_v0_5_0_context_preallocated_clone (cannot be NULL)
+ *               rustsecp256k1zkp_v0_6_0_context_preallocated_create or
+ *               rustsecp256k1zkp_v0_6_0_context_preallocated_clone.
  */
-SECP256K1_API void rustsecp256k1zkp_v0_5_0_context_preallocated_destroy(
-    rustsecp256k1zkp_v0_5_0_context* ctx
-);
+SECP256K1_API void rustsecp256k1zkp_v0_6_0_context_preallocated_destroy(
+    rustsecp256k1zkp_v0_6_0_context* ctx
+) SECP256K1_ARG_NONNULL(1);
 
 #ifdef __cplusplus
 }
