@@ -28,7 +28,7 @@
  *  sage: '%x' % (EllipticCurve ([F (a), F (b)]).order())
  *   'fffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141'
  */
-static const rustsecp256k1zkp_v0_5_0_fe rustsecp256k1zkp_v0_5_0_ecdsa_const_order_as_fe = SECP256K1_FE_CONST(
+static const rustsecp256k1zkp_v0_6_0_fe rustsecp256k1zkp_v0_6_0_ecdsa_const_order_as_fe = SECP256K1_FE_CONST(
     0xFFFFFFFFUL, 0xFFFFFFFFUL, 0xFFFFFFFFUL, 0xFFFFFFFEUL,
     0xBAAEDCE6UL, 0xAF48A03BUL, 0xBFD25E8CUL, 0xD0364141UL
 );
@@ -42,11 +42,11 @@ static const rustsecp256k1zkp_v0_5_0_fe rustsecp256k1zkp_v0_5_0_ecdsa_const_orde
  *  sage: '%x' % (p - EllipticCurve ([F (a), F (b)]).order())
  *   '14551231950b75fc4402da1722fc9baee'
  */
-static const rustsecp256k1zkp_v0_5_0_fe rustsecp256k1zkp_v0_5_0_ecdsa_const_p_minus_order = SECP256K1_FE_CONST(
+static const rustsecp256k1zkp_v0_6_0_fe rustsecp256k1zkp_v0_6_0_ecdsa_const_p_minus_order = SECP256K1_FE_CONST(
     0, 0, 0, 1, 0x45512319UL, 0x50B75FC4UL, 0x402DA172UL, 0x2FC9BAEEUL
 );
 
-static int rustsecp256k1zkp_v0_5_0_der_read_len(size_t *len, const unsigned char **sigp, const unsigned char *sigend) {
+static int rustsecp256k1zkp_v0_6_0_der_read_len(size_t *len, const unsigned char **sigp, const unsigned char *sigend) {
     size_t lenleft;
     unsigned char b1;
     VERIFY_CHECK(len != NULL);
@@ -99,7 +99,7 @@ static int rustsecp256k1zkp_v0_5_0_der_read_len(size_t *len, const unsigned char
     return 1;
 }
 
-static int rustsecp256k1zkp_v0_5_0_der_parse_integer(rustsecp256k1zkp_v0_5_0_scalar *r, const unsigned char **sig, const unsigned char *sigend) {
+static int rustsecp256k1zkp_v0_6_0_der_parse_integer(rustsecp256k1zkp_v0_6_0_scalar *r, const unsigned char **sig, const unsigned char *sigend) {
     int overflow = 0;
     unsigned char ra[32] = {0};
     size_t rlen;
@@ -109,10 +109,10 @@ static int rustsecp256k1zkp_v0_5_0_der_parse_integer(rustsecp256k1zkp_v0_5_0_sca
         return 0;
     }
     (*sig)++;
-    if (rustsecp256k1zkp_v0_5_0_der_read_len(&rlen, sig, sigend) == 0) {
+    if (rustsecp256k1zkp_v0_6_0_der_read_len(&rlen, sig, sigend) == 0) {
         return 0;
     }
-    if (rlen == 0 || *sig + rlen > sigend) {
+    if (rlen == 0 || rlen > (size_t)(sigend - *sig)) {
         /* Exceeds bounds or not at least length 1 (X.690-0207 8.3.1).  */
         return 0;
     }
@@ -140,24 +140,24 @@ static int rustsecp256k1zkp_v0_5_0_der_parse_integer(rustsecp256k1zkp_v0_5_0_sca
         overflow = 1;
     }
     if (!overflow) {
-        memcpy(ra + 32 - rlen, *sig, rlen);
-        rustsecp256k1zkp_v0_5_0_scalar_set_b32(r, ra, &overflow);
+        if (rlen) memcpy(ra + 32 - rlen, *sig, rlen);
+        rustsecp256k1zkp_v0_6_0_scalar_set_b32(r, ra, &overflow);
     }
     if (overflow) {
-        rustsecp256k1zkp_v0_5_0_scalar_set_int(r, 0);
+        rustsecp256k1zkp_v0_6_0_scalar_set_int(r, 0);
     }
     (*sig) += rlen;
     return 1;
 }
 
-static int rustsecp256k1zkp_v0_5_0_ecdsa_sig_parse(rustsecp256k1zkp_v0_5_0_scalar *rr, rustsecp256k1zkp_v0_5_0_scalar *rs, const unsigned char *sig, size_t size) {
+static int rustsecp256k1zkp_v0_6_0_ecdsa_sig_parse(rustsecp256k1zkp_v0_6_0_scalar *rr, rustsecp256k1zkp_v0_6_0_scalar *rs, const unsigned char *sig, size_t size) {
     const unsigned char *sigend = sig + size;
     size_t rlen;
     if (sig == sigend || *(sig++) != 0x30) {
         /* The encoding doesn't start with a constructed sequence (X.690-0207 8.9.1). */
         return 0;
     }
-    if (rustsecp256k1zkp_v0_5_0_der_read_len(&rlen, &sig, sigend) == 0) {
+    if (rustsecp256k1zkp_v0_6_0_der_read_len(&rlen, &sig, sigend) == 0) {
         return 0;
     }
     if (rlen != (size_t)(sigend - sig)) {
@@ -165,10 +165,10 @@ static int rustsecp256k1zkp_v0_5_0_ecdsa_sig_parse(rustsecp256k1zkp_v0_5_0_scala
         return 0;
     }
 
-    if (!rustsecp256k1zkp_v0_5_0_der_parse_integer(rr, &sig, sigend)) {
+    if (!rustsecp256k1zkp_v0_6_0_der_parse_integer(rr, &sig, sigend)) {
         return 0;
     }
-    if (!rustsecp256k1zkp_v0_5_0_der_parse_integer(rs, &sig, sigend)) {
+    if (!rustsecp256k1zkp_v0_6_0_der_parse_integer(rs, &sig, sigend)) {
         return 0;
     }
 
@@ -180,12 +180,12 @@ static int rustsecp256k1zkp_v0_5_0_ecdsa_sig_parse(rustsecp256k1zkp_v0_5_0_scala
     return 1;
 }
 
-static int rustsecp256k1zkp_v0_5_0_ecdsa_sig_serialize(unsigned char *sig, size_t *size, const rustsecp256k1zkp_v0_5_0_scalar* ar, const rustsecp256k1zkp_v0_5_0_scalar* as) {
+static int rustsecp256k1zkp_v0_6_0_ecdsa_sig_serialize(unsigned char *sig, size_t *size, const rustsecp256k1zkp_v0_6_0_scalar* ar, const rustsecp256k1zkp_v0_6_0_scalar* as) {
     unsigned char r[33] = {0}, s[33] = {0};
     unsigned char *rp = r, *sp = s;
     size_t lenR = 33, lenS = 33;
-    rustsecp256k1zkp_v0_5_0_scalar_get_b32(&r[1], ar);
-    rustsecp256k1zkp_v0_5_0_scalar_get_b32(&s[1], as);
+    rustsecp256k1zkp_v0_6_0_scalar_get_b32(&r[1], ar);
+    rustsecp256k1zkp_v0_6_0_scalar_get_b32(&s[1], as);
     while (lenR > 1 && rp[0] == 0 && rp[1] < 0x80) { lenR--; rp++; }
     while (lenS > 1 && sp[0] == 0 && sp[1] < 0x80) { lenS--; sp++; }
     if (*size < 6+lenS+lenR) {
@@ -204,42 +204,42 @@ static int rustsecp256k1zkp_v0_5_0_ecdsa_sig_serialize(unsigned char *sig, size_
     return 1;
 }
 
-static int rustsecp256k1zkp_v0_5_0_ecdsa_sig_verify(const rustsecp256k1zkp_v0_5_0_ecmult_context *ctx, const rustsecp256k1zkp_v0_5_0_scalar *sigr, const rustsecp256k1zkp_v0_5_0_scalar *sigs, const rustsecp256k1zkp_v0_5_0_ge *pubkey, const rustsecp256k1zkp_v0_5_0_scalar *message) {
+static int rustsecp256k1zkp_v0_6_0_ecdsa_sig_verify(const rustsecp256k1zkp_v0_6_0_scalar *sigr, const rustsecp256k1zkp_v0_6_0_scalar *sigs, const rustsecp256k1zkp_v0_6_0_ge *pubkey, const rustsecp256k1zkp_v0_6_0_scalar *message) {
     unsigned char c[32];
-    rustsecp256k1zkp_v0_5_0_scalar sn, u1, u2;
+    rustsecp256k1zkp_v0_6_0_scalar sn, u1, u2;
 #if !defined(EXHAUSTIVE_TEST_ORDER)
-    rustsecp256k1zkp_v0_5_0_fe xr;
+    rustsecp256k1zkp_v0_6_0_fe xr;
 #endif
-    rustsecp256k1zkp_v0_5_0_gej pubkeyj;
-    rustsecp256k1zkp_v0_5_0_gej pr;
+    rustsecp256k1zkp_v0_6_0_gej pubkeyj;
+    rustsecp256k1zkp_v0_6_0_gej pr;
 
-    if (rustsecp256k1zkp_v0_5_0_scalar_is_zero(sigr) || rustsecp256k1zkp_v0_5_0_scalar_is_zero(sigs)) {
+    if (rustsecp256k1zkp_v0_6_0_scalar_is_zero(sigr) || rustsecp256k1zkp_v0_6_0_scalar_is_zero(sigs)) {
         return 0;
     }
 
-    rustsecp256k1zkp_v0_5_0_scalar_inverse_var(&sn, sigs);
-    rustsecp256k1zkp_v0_5_0_scalar_mul(&u1, &sn, message);
-    rustsecp256k1zkp_v0_5_0_scalar_mul(&u2, &sn, sigr);
-    rustsecp256k1zkp_v0_5_0_gej_set_ge(&pubkeyj, pubkey);
-    rustsecp256k1zkp_v0_5_0_ecmult(ctx, &pr, &pubkeyj, &u2, &u1);
-    if (rustsecp256k1zkp_v0_5_0_gej_is_infinity(&pr)) {
+    rustsecp256k1zkp_v0_6_0_scalar_inverse_var(&sn, sigs);
+    rustsecp256k1zkp_v0_6_0_scalar_mul(&u1, &sn, message);
+    rustsecp256k1zkp_v0_6_0_scalar_mul(&u2, &sn, sigr);
+    rustsecp256k1zkp_v0_6_0_gej_set_ge(&pubkeyj, pubkey);
+    rustsecp256k1zkp_v0_6_0_ecmult(&pr, &pubkeyj, &u2, &u1);
+    if (rustsecp256k1zkp_v0_6_0_gej_is_infinity(&pr)) {
         return 0;
     }
 
 #if defined(EXHAUSTIVE_TEST_ORDER)
 {
-    rustsecp256k1zkp_v0_5_0_scalar computed_r;
-    rustsecp256k1zkp_v0_5_0_ge pr_ge;
-    rustsecp256k1zkp_v0_5_0_ge_set_gej(&pr_ge, &pr);
-    rustsecp256k1zkp_v0_5_0_fe_normalize(&pr_ge.x);
+    rustsecp256k1zkp_v0_6_0_scalar computed_r;
+    rustsecp256k1zkp_v0_6_0_ge pr_ge;
+    rustsecp256k1zkp_v0_6_0_ge_set_gej(&pr_ge, &pr);
+    rustsecp256k1zkp_v0_6_0_fe_normalize(&pr_ge.x);
 
-    rustsecp256k1zkp_v0_5_0_fe_get_b32(c, &pr_ge.x);
-    rustsecp256k1zkp_v0_5_0_scalar_set_b32(&computed_r, c, NULL);
-    return rustsecp256k1zkp_v0_5_0_scalar_eq(sigr, &computed_r);
+    rustsecp256k1zkp_v0_6_0_fe_get_b32(c, &pr_ge.x);
+    rustsecp256k1zkp_v0_6_0_scalar_set_b32(&computed_r, c, NULL);
+    return rustsecp256k1zkp_v0_6_0_scalar_eq(sigr, &computed_r);
 }
 #else
-    rustsecp256k1zkp_v0_5_0_scalar_get_b32(c, sigr);
-    rustsecp256k1zkp_v0_5_0_fe_set_b32(&xr, c);
+    rustsecp256k1zkp_v0_6_0_scalar_get_b32(c, sigr);
+    rustsecp256k1zkp_v0_6_0_fe_set_b32(&xr, c);
 
     /** We now have the recomputed R point in pr, and its claimed x coordinate (modulo n)
      *  in xr. Naively, we would extract the x coordinate from pr (requiring a inversion modulo p),
@@ -255,18 +255,18 @@ static int rustsecp256k1zkp_v0_5_0_ecdsa_sig_verify(const rustsecp256k1zkp_v0_5_
      *    <=> (xr * pr.z^2 mod p == pr.x) || (xr + n < p && (xr + n) * pr.z^2 mod p == pr.x)
      *
      *  Thus, we can avoid the inversion, but we have to check both cases separately.
-     *  rustsecp256k1zkp_v0_5_0_gej_eq_x implements the (xr * pr.z^2 mod p == pr.x) test.
+     *  rustsecp256k1zkp_v0_6_0_gej_eq_x implements the (xr * pr.z^2 mod p == pr.x) test.
      */
-    if (rustsecp256k1zkp_v0_5_0_gej_eq_x_var(&xr, &pr)) {
+    if (rustsecp256k1zkp_v0_6_0_gej_eq_x_var(&xr, &pr)) {
         /* xr * pr.z^2 mod p == pr.x, so the signature is valid. */
         return 1;
     }
-    if (rustsecp256k1zkp_v0_5_0_fe_cmp_var(&xr, &rustsecp256k1zkp_v0_5_0_ecdsa_const_p_minus_order) >= 0) {
+    if (rustsecp256k1zkp_v0_6_0_fe_cmp_var(&xr, &rustsecp256k1zkp_v0_6_0_ecdsa_const_p_minus_order) >= 0) {
         /* xr + n >= p, so we can skip testing the second case. */
         return 0;
     }
-    rustsecp256k1zkp_v0_5_0_fe_add(&xr, &rustsecp256k1zkp_v0_5_0_ecdsa_const_order_as_fe);
-    if (rustsecp256k1zkp_v0_5_0_gej_eq_x_var(&xr, &pr)) {
+    rustsecp256k1zkp_v0_6_0_fe_add(&xr, &rustsecp256k1zkp_v0_6_0_ecdsa_const_order_as_fe);
+    if (rustsecp256k1zkp_v0_6_0_gej_eq_x_var(&xr, &pr)) {
         /* (xr + n) * pr.z^2 mod p == pr.x, so the signature is valid. */
         return 1;
     }
@@ -274,42 +274,42 @@ static int rustsecp256k1zkp_v0_5_0_ecdsa_sig_verify(const rustsecp256k1zkp_v0_5_
 #endif
 }
 
-static int rustsecp256k1zkp_v0_5_0_ecdsa_sig_sign(const rustsecp256k1zkp_v0_5_0_ecmult_gen_context *ctx, rustsecp256k1zkp_v0_5_0_scalar *sigr, rustsecp256k1zkp_v0_5_0_scalar *sigs, const rustsecp256k1zkp_v0_5_0_scalar *seckey, const rustsecp256k1zkp_v0_5_0_scalar *message, const rustsecp256k1zkp_v0_5_0_scalar *nonce, int *recid) {
+static int rustsecp256k1zkp_v0_6_0_ecdsa_sig_sign(const rustsecp256k1zkp_v0_6_0_ecmult_gen_context *ctx, rustsecp256k1zkp_v0_6_0_scalar *sigr, rustsecp256k1zkp_v0_6_0_scalar *sigs, const rustsecp256k1zkp_v0_6_0_scalar *seckey, const rustsecp256k1zkp_v0_6_0_scalar *message, const rustsecp256k1zkp_v0_6_0_scalar *nonce, int *recid) {
     unsigned char b[32];
-    rustsecp256k1zkp_v0_5_0_gej rp;
-    rustsecp256k1zkp_v0_5_0_ge r;
-    rustsecp256k1zkp_v0_5_0_scalar n;
+    rustsecp256k1zkp_v0_6_0_gej rp;
+    rustsecp256k1zkp_v0_6_0_ge r;
+    rustsecp256k1zkp_v0_6_0_scalar n;
     int overflow = 0;
     int high;
 
-    rustsecp256k1zkp_v0_5_0_ecmult_gen(ctx, &rp, nonce);
-    rustsecp256k1zkp_v0_5_0_ge_set_gej(&r, &rp);
-    rustsecp256k1zkp_v0_5_0_fe_normalize(&r.x);
-    rustsecp256k1zkp_v0_5_0_fe_normalize(&r.y);
-    rustsecp256k1zkp_v0_5_0_fe_get_b32(b, &r.x);
-    rustsecp256k1zkp_v0_5_0_scalar_set_b32(sigr, b, &overflow);
+    rustsecp256k1zkp_v0_6_0_ecmult_gen(ctx, &rp, nonce);
+    rustsecp256k1zkp_v0_6_0_ge_set_gej(&r, &rp);
+    rustsecp256k1zkp_v0_6_0_fe_normalize(&r.x);
+    rustsecp256k1zkp_v0_6_0_fe_normalize(&r.y);
+    rustsecp256k1zkp_v0_6_0_fe_get_b32(b, &r.x);
+    rustsecp256k1zkp_v0_6_0_scalar_set_b32(sigr, b, &overflow);
     if (recid) {
         /* The overflow condition is cryptographically unreachable as hitting it requires finding the discrete log
          * of some P where P.x >= order, and only 1 in about 2^127 points meet this criteria.
          */
-        *recid = (overflow << 1) | rustsecp256k1zkp_v0_5_0_fe_is_odd(&r.y);
+        *recid = (overflow << 1) | rustsecp256k1zkp_v0_6_0_fe_is_odd(&r.y);
     }
-    rustsecp256k1zkp_v0_5_0_scalar_mul(&n, sigr, seckey);
-    rustsecp256k1zkp_v0_5_0_scalar_add(&n, &n, message);
-    rustsecp256k1zkp_v0_5_0_scalar_inverse(sigs, nonce);
-    rustsecp256k1zkp_v0_5_0_scalar_mul(sigs, sigs, &n);
-    rustsecp256k1zkp_v0_5_0_scalar_clear(&n);
-    rustsecp256k1zkp_v0_5_0_gej_clear(&rp);
-    rustsecp256k1zkp_v0_5_0_ge_clear(&r);
-    high = rustsecp256k1zkp_v0_5_0_scalar_is_high(sigs);
-    rustsecp256k1zkp_v0_5_0_scalar_cond_negate(sigs, high);
+    rustsecp256k1zkp_v0_6_0_scalar_mul(&n, sigr, seckey);
+    rustsecp256k1zkp_v0_6_0_scalar_add(&n, &n, message);
+    rustsecp256k1zkp_v0_6_0_scalar_inverse(sigs, nonce);
+    rustsecp256k1zkp_v0_6_0_scalar_mul(sigs, sigs, &n);
+    rustsecp256k1zkp_v0_6_0_scalar_clear(&n);
+    rustsecp256k1zkp_v0_6_0_gej_clear(&rp);
+    rustsecp256k1zkp_v0_6_0_ge_clear(&r);
+    high = rustsecp256k1zkp_v0_6_0_scalar_is_high(sigs);
+    rustsecp256k1zkp_v0_6_0_scalar_cond_negate(sigs, high);
     if (recid) {
-            *recid ^= high;
+        *recid ^= high;
     }
     /* P.x = order is on the curve, so technically sig->r could end up being zero, which would be an invalid signature.
      * This is cryptographically unreachable as hitting it requires finding the discrete log of P.x = N.
      */
-    return !rustsecp256k1zkp_v0_5_0_scalar_is_zero(sigr) & !rustsecp256k1zkp_v0_5_0_scalar_is_zero(sigs);
+    return (int)(!rustsecp256k1zkp_v0_6_0_scalar_is_zero(sigr)) & (int)(!rustsecp256k1zkp_v0_6_0_scalar_is_zero(sigs));
 }
 
 #endif /* SECP256K1_ECDSA_IMPL_H */
