@@ -9,9 +9,19 @@
 
 #include "util.h"
 #include "field.h"
+#include "modinv32_impl.h"
+
+/** See the comment at the top of field_5x52_impl.h for more details.
+ *
+ *  Here, we represent field elements as 10 uint32_t's in base 2^26, least significant first,
+ *  where limbs can contain >26 bits.
+ *  A magnitude M means:
+ *  - 2*M*(2^22-1) is the max (inclusive) of the most significant limb
+ *  - 2*M*(2^26-1) is the max (inclusive) of the remaining limbs
+ */
 
 #ifdef VERIFY
-static void rustsecp256k1zkp_v0_5_0_fe_verify(const rustsecp256k1zkp_v0_5_0_fe *a) {
+static void rustsecp256k1zkp_v0_6_0_fe_verify(const rustsecp256k1zkp_v0_6_0_fe *a) {
     const uint32_t *d = a->n;
     int m = a->normalized ? 1 : 2 * a->magnitude, r = 1;
     r &= (d[0] <= 0x3FFFFFFUL * m);
@@ -39,7 +49,7 @@ static void rustsecp256k1zkp_v0_5_0_fe_verify(const rustsecp256k1zkp_v0_5_0_fe *
 }
 #endif
 
-static void rustsecp256k1zkp_v0_5_0_fe_normalize(rustsecp256k1zkp_v0_5_0_fe *r) {
+static void rustsecp256k1zkp_v0_6_0_fe_normalize(rustsecp256k1zkp_v0_6_0_fe *r) {
     uint32_t t0 = r->n[0], t1 = r->n[1], t2 = r->n[2], t3 = r->n[3], t4 = r->n[4],
              t5 = r->n[5], t6 = r->n[6], t7 = r->n[7], t8 = r->n[8], t9 = r->n[9];
 
@@ -90,11 +100,11 @@ static void rustsecp256k1zkp_v0_5_0_fe_normalize(rustsecp256k1zkp_v0_5_0_fe *r)
 #ifdef VERIFY
     r->magnitude = 1;
     r->normalized = 1;
-    rustsecp256k1zkp_v0_5_0_fe_verify(r);
+    rustsecp256k1zkp_v0_6_0_fe_verify(r);
 #endif
 }
 
-static void rustsecp256k1zkp_v0_5_0_fe_normalize_weak(rustsecp256k1zkp_v0_5_0_fe *r) {
+static void rustsecp256k1zkp_v0_6_0_fe_normalize_weak(rustsecp256k1zkp_v0_6_0_fe *r) {
     uint32_t t0 = r->n[0], t1 = r->n[1], t2 = r->n[2], t3 = r->n[3], t4 = r->n[4],
              t5 = r->n[5], t6 = r->n[6], t7 = r->n[7], t8 = r->n[8], t9 = r->n[9];
 
@@ -121,11 +131,11 @@ static void rustsecp256k1zkp_v0_5_0_fe_normalize_weak(rustsecp256k1zkp_v0_5_0_fe
 
 #ifdef VERIFY
     r->magnitude = 1;
-    rustsecp256k1zkp_v0_5_0_fe_verify(r);
+    rustsecp256k1zkp_v0_6_0_fe_verify(r);
 #endif
 }
 
-static void rustsecp256k1zkp_v0_5_0_fe_normalize_var(rustsecp256k1zkp_v0_5_0_fe *r) {
+static void rustsecp256k1zkp_v0_6_0_fe_normalize_var(rustsecp256k1zkp_v0_6_0_fe *r) {
     uint32_t t0 = r->n[0], t1 = r->n[1], t2 = r->n[2], t3 = r->n[3], t4 = r->n[4],
              t5 = r->n[5], t6 = r->n[6], t7 = r->n[7], t8 = r->n[8], t9 = r->n[9];
 
@@ -177,11 +187,11 @@ static void rustsecp256k1zkp_v0_5_0_fe_normalize_var(rustsecp256k1zkp_v0_5_0_fe
 #ifdef VERIFY
     r->magnitude = 1;
     r->normalized = 1;
-    rustsecp256k1zkp_v0_5_0_fe_verify(r);
+    rustsecp256k1zkp_v0_6_0_fe_verify(r);
 #endif
 }
 
-static int rustsecp256k1zkp_v0_5_0_fe_normalizes_to_zero(rustsecp256k1zkp_v0_5_0_fe *r) {
+static int rustsecp256k1zkp_v0_6_0_fe_normalizes_to_zero(const rustsecp256k1zkp_v0_6_0_fe *r) {
     uint32_t t0 = r->n[0], t1 = r->n[1], t2 = r->n[2], t3 = r->n[3], t4 = r->n[4],
              t5 = r->n[5], t6 = r->n[6], t7 = r->n[7], t8 = r->n[8], t9 = r->n[9];
 
@@ -210,7 +220,7 @@ static int rustsecp256k1zkp_v0_5_0_fe_normalizes_to_zero(rustsecp256k1zkp_v0_5_0
     return (z0 == 0) | (z1 == 0x3FFFFFFUL);
 }
 
-static int rustsecp256k1zkp_v0_5_0_fe_normalizes_to_zero_var(rustsecp256k1zkp_v0_5_0_fe *r) {
+static int rustsecp256k1zkp_v0_6_0_fe_normalizes_to_zero_var(const rustsecp256k1zkp_v0_6_0_fe *r) {
     uint32_t t0, t1, t2, t3, t4, t5, t6, t7, t8, t9;
     uint32_t z0, z1;
     uint32_t x;
@@ -262,34 +272,35 @@ static int rustsecp256k1zkp_v0_5_0_fe_normalizes_to_zero_var(rustsecp256k1zkp_v0
     return (z0 == 0) | (z1 == 0x3FFFFFFUL);
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_set_int(rustsecp256k1zkp_v0_5_0_fe *r, int a) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_fe_set_int(rustsecp256k1zkp_v0_6_0_fe *r, int a) {
+    VERIFY_CHECK(0 <= a && a <= 0x7FFF);
     r->n[0] = a;
     r->n[1] = r->n[2] = r->n[3] = r->n[4] = r->n[5] = r->n[6] = r->n[7] = r->n[8] = r->n[9] = 0;
 #ifdef VERIFY
-    r->magnitude = 1;
+    r->magnitude = (a != 0);
     r->normalized = 1;
-    rustsecp256k1zkp_v0_5_0_fe_verify(r);
+    rustsecp256k1zkp_v0_6_0_fe_verify(r);
 #endif
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_fe_is_zero(const rustsecp256k1zkp_v0_5_0_fe *a) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_fe_is_zero(const rustsecp256k1zkp_v0_6_0_fe *a) {
     const uint32_t *t = a->n;
 #ifdef VERIFY
     VERIFY_CHECK(a->normalized);
-    rustsecp256k1zkp_v0_5_0_fe_verify(a);
+    rustsecp256k1zkp_v0_6_0_fe_verify(a);
 #endif
     return (t[0] | t[1] | t[2] | t[3] | t[4] | t[5] | t[6] | t[7] | t[8] | t[9]) == 0;
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_fe_is_odd(const rustsecp256k1zkp_v0_5_0_fe *a) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_fe_is_odd(const rustsecp256k1zkp_v0_6_0_fe *a) {
 #ifdef VERIFY
     VERIFY_CHECK(a->normalized);
-    rustsecp256k1zkp_v0_5_0_fe_verify(a);
+    rustsecp256k1zkp_v0_6_0_fe_verify(a);
 #endif
     return a->n[0] & 1;
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_clear(rustsecp256k1zkp_v0_5_0_fe *a) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_fe_clear(rustsecp256k1zkp_v0_6_0_fe *a) {
     int i;
 #ifdef VERIFY
     a->magnitude = 0;
@@ -300,13 +311,13 @@ SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_clear(rustsecp256k1zkp_v
     }
 }
 
-static int rustsecp256k1zkp_v0_5_0_fe_cmp_var(const rustsecp256k1zkp_v0_5_0_fe *a, const rustsecp256k1zkp_v0_5_0_fe *b) {
+static int rustsecp256k1zkp_v0_6_0_fe_cmp_var(const rustsecp256k1zkp_v0_6_0_fe *a, const rustsecp256k1zkp_v0_6_0_fe *b) {
     int i;
 #ifdef VERIFY
     VERIFY_CHECK(a->normalized);
     VERIFY_CHECK(b->normalized);
-    rustsecp256k1zkp_v0_5_0_fe_verify(a);
-    rustsecp256k1zkp_v0_5_0_fe_verify(b);
+    rustsecp256k1zkp_v0_6_0_fe_verify(a);
+    rustsecp256k1zkp_v0_6_0_fe_verify(b);
 #endif
     for (i = 9; i >= 0; i--) {
         if (a->n[i] > b->n[i]) {
@@ -319,7 +330,7 @@ static int rustsecp256k1zkp_v0_5_0_fe_cmp_var(const rustsecp256k1zkp_v0_5_0_fe *
     return 0;
 }
 
-static int rustsecp256k1zkp_v0_5_0_fe_set_b32(rustsecp256k1zkp_v0_5_0_fe *r, const unsigned char *a) {
+static int rustsecp256k1zkp_v0_6_0_fe_set_b32(rustsecp256k1zkp_v0_6_0_fe *r, const unsigned char *a) {
     int ret;
     r->n[0] = (uint32_t)a[31] | ((uint32_t)a[30] << 8) | ((uint32_t)a[29] << 16) | ((uint32_t)(a[28] & 0x3) << 24);
     r->n[1] = (uint32_t)((a[28] >> 2) & 0x3f) | ((uint32_t)a[27] << 6) | ((uint32_t)a[26] << 14) | ((uint32_t)(a[25] & 0xf) << 22);
@@ -337,7 +348,7 @@ static int rustsecp256k1zkp_v0_5_0_fe_set_b32(rustsecp256k1zkp_v0_5_0_fe *r, con
     r->magnitude = 1;
     if (ret) {
         r->normalized = 1;
-        rustsecp256k1zkp_v0_5_0_fe_verify(r);
+        rustsecp256k1zkp_v0_6_0_fe_verify(r);
     } else {
         r->normalized = 0;
     }
@@ -346,10 +357,10 @@ static int rustsecp256k1zkp_v0_5_0_fe_set_b32(rustsecp256k1zkp_v0_5_0_fe *r, con
 }
 
 /** Convert a field element to a 32-byte big endian value. Requires the input to be normalized */
-static void rustsecp256k1zkp_v0_5_0_fe_get_b32(unsigned char *r, const rustsecp256k1zkp_v0_5_0_fe *a) {
+static void rustsecp256k1zkp_v0_6_0_fe_get_b32(unsigned char *r, const rustsecp256k1zkp_v0_6_0_fe *a) {
 #ifdef VERIFY
     VERIFY_CHECK(a->normalized);
-    rustsecp256k1zkp_v0_5_0_fe_verify(a);
+    rustsecp256k1zkp_v0_6_0_fe_verify(a);
 #endif
     r[0] = (a->n[9] >> 14) & 0xff;
     r[1] = (a->n[9] >> 6) & 0xff;
@@ -385,10 +396,14 @@ static void rustsecp256k1zkp_v0_5_0_fe_get_b32(unsigned char *r, const rustsecp2
     r[31] = a->n[0] & 0xff;
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_negate(rustsecp256k1zkp_v0_5_0_fe *r, const rustsecp256k1zkp_v0_5_0_fe *a, int m) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_fe_negate(rustsecp256k1zkp_v0_6_0_fe *r, const rustsecp256k1zkp_v0_6_0_fe *a, int m) {
 #ifdef VERIFY
     VERIFY_CHECK(a->magnitude <= m);
-    rustsecp256k1zkp_v0_5_0_fe_verify(a);
+    rustsecp256k1zkp_v0_6_0_fe_verify(a);
+    VERIFY_CHECK(0x3FFFC2FUL * 2 * (m + 1) >= 0x3FFFFFFUL * 2 * m);
+    VERIFY_CHECK(0x3FFFFBFUL * 2 * (m + 1) >= 0x3FFFFFFUL * 2 * m);
+    VERIFY_CHECK(0x3FFFFFFUL * 2 * (m + 1) >= 0x3FFFFFFUL * 2 * m);
+    VERIFY_CHECK(0x03FFFFFUL * 2 * (m + 1) >= 0x03FFFFFUL * 2 * m);
 #endif
     r->n[0] = 0x3FFFC2FUL * 2 * (m + 1) - a->n[0];
     r->n[1] = 0x3FFFFBFUL * 2 * (m + 1) - a->n[1];
@@ -403,11 +418,11 @@ SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_negate(rustsecp256k1zkp_
 #ifdef VERIFY
     r->magnitude = m + 1;
     r->normalized = 0;
-    rustsecp256k1zkp_v0_5_0_fe_verify(r);
+    rustsecp256k1zkp_v0_6_0_fe_verify(r);
 #endif
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_mul_int(rustsecp256k1zkp_v0_5_0_fe *r, int a) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_fe_mul_int(rustsecp256k1zkp_v0_6_0_fe *r, int a) {
     r->n[0] *= a;
     r->n[1] *= a;
     r->n[2] *= a;
@@ -421,13 +436,13 @@ SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_mul_int(rustsecp256k1zkp
 #ifdef VERIFY
     r->magnitude *= a;
     r->normalized = 0;
-    rustsecp256k1zkp_v0_5_0_fe_verify(r);
+    rustsecp256k1zkp_v0_6_0_fe_verify(r);
 #endif
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_add(rustsecp256k1zkp_v0_5_0_fe *r, const rustsecp256k1zkp_v0_5_0_fe *a) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_fe_add(rustsecp256k1zkp_v0_6_0_fe *r, const rustsecp256k1zkp_v0_6_0_fe *a) {
 #ifdef VERIFY
-    rustsecp256k1zkp_v0_5_0_fe_verify(a);
+    rustsecp256k1zkp_v0_6_0_fe_verify(a);
 #endif
     r->n[0] += a->n[0];
     r->n[1] += a->n[1];
@@ -442,15 +457,15 @@ SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_add(rustsecp256k1zkp_v0_
 #ifdef VERIFY
     r->magnitude += a->magnitude;
     r->normalized = 0;
-    rustsecp256k1zkp_v0_5_0_fe_verify(r);
+    rustsecp256k1zkp_v0_6_0_fe_verify(r);
 #endif
 }
 
 #if defined(USE_EXTERNAL_ASM)
 
 /* External assembler implementation */
-void rustsecp256k1zkp_v0_5_0_fe_mul_inner(uint32_t *r, const uint32_t *a, const uint32_t * SECP256K1_RESTRICT b);
-void rustsecp256k1zkp_v0_5_0_fe_sqr_inner(uint32_t *r, const uint32_t *a);
+void rustsecp256k1zkp_v0_6_0_fe_mul_inner(uint32_t *r, const uint32_t *a, const uint32_t * SECP256K1_RESTRICT b);
+void rustsecp256k1zkp_v0_6_0_fe_sqr_inner(uint32_t *r, const uint32_t *a);
 
 #else
 
@@ -460,7 +475,7 @@ void rustsecp256k1zkp_v0_5_0_fe_sqr_inner(uint32_t *r, const uint32_t *a);
 #define VERIFY_BITS(x, n) do { } while(0)
 #endif
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_mul_inner(uint32_t *r, const uint32_t *a, const uint32_t * SECP256K1_RESTRICT b) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_fe_mul_inner(uint32_t *r, const uint32_t *a, const uint32_t * SECP256K1_RESTRICT b) {
     uint64_t c, d;
     uint64_t u0, u1, u2, u3, u4, u5, u6, u7, u8;
     uint32_t t9, t1, t0, t2, t3, t4, t5, t6, t7;
@@ -790,7 +805,7 @@ SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_mul_inner(uint32_t *r, c
     /* [r9 r8 r7 r6 r5 r4 r3 r2 r1 r0] = [p18 p17 p16 p15 p14 p13 p12 p11 p10 p9 p8 p7 p6 p5 p4 p3 p2 p1 p0] */
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_sqr_inner(uint32_t *r, const uint32_t *a) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_fe_sqr_inner(uint32_t *r, const uint32_t *a) {
     uint64_t c, d;
     uint64_t u0, u1, u2, u3, u4, u5, u6, u7, u8;
     uint32_t t9, t0, t1, t2, t3, t4, t5, t6, t7;
@@ -1065,37 +1080,37 @@ SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_fe_sqr_inner(uint32_t *r, c
 }
 #endif
 
-static void rustsecp256k1zkp_v0_5_0_fe_mul(rustsecp256k1zkp_v0_5_0_fe *r, const rustsecp256k1zkp_v0_5_0_fe *a, const rustsecp256k1zkp_v0_5_0_fe * SECP256K1_RESTRICT b) {
+static void rustsecp256k1zkp_v0_6_0_fe_mul(rustsecp256k1zkp_v0_6_0_fe *r, const rustsecp256k1zkp_v0_6_0_fe *a, const rustsecp256k1zkp_v0_6_0_fe * SECP256K1_RESTRICT b) {
 #ifdef VERIFY
     VERIFY_CHECK(a->magnitude <= 8);
     VERIFY_CHECK(b->magnitude <= 8);
-    rustsecp256k1zkp_v0_5_0_fe_verify(a);
-    rustsecp256k1zkp_v0_5_0_fe_verify(b);
+    rustsecp256k1zkp_v0_6_0_fe_verify(a);
+    rustsecp256k1zkp_v0_6_0_fe_verify(b);
     VERIFY_CHECK(r != b);
     VERIFY_CHECK(a != b);
 #endif
-    rustsecp256k1zkp_v0_5_0_fe_mul_inner(r->n, a->n, b->n);
+    rustsecp256k1zkp_v0_6_0_fe_mul_inner(r->n, a->n, b->n);
 #ifdef VERIFY
     r->magnitude = 1;
     r->normalized = 0;
-    rustsecp256k1zkp_v0_5_0_fe_verify(r);
+    rustsecp256k1zkp_v0_6_0_fe_verify(r);
 #endif
 }
 
-static void rustsecp256k1zkp_v0_5_0_fe_sqr(rustsecp256k1zkp_v0_5_0_fe *r, const rustsecp256k1zkp_v0_5_0_fe *a) {
+static void rustsecp256k1zkp_v0_6_0_fe_sqr(rustsecp256k1zkp_v0_6_0_fe *r, const rustsecp256k1zkp_v0_6_0_fe *a) {
 #ifdef VERIFY
     VERIFY_CHECK(a->magnitude <= 8);
-    rustsecp256k1zkp_v0_5_0_fe_verify(a);
+    rustsecp256k1zkp_v0_6_0_fe_verify(a);
 #endif
-    rustsecp256k1zkp_v0_5_0_fe_sqr_inner(r->n, a->n);
+    rustsecp256k1zkp_v0_6_0_fe_sqr_inner(r->n, a->n);
 #ifdef VERIFY
     r->magnitude = 1;
     r->normalized = 0;
-    rustsecp256k1zkp_v0_5_0_fe_verify(r);
+    rustsecp256k1zkp_v0_6_0_fe_verify(r);
 #endif
 }
 
-static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_fe_cmov(rustsecp256k1zkp_v0_5_0_fe *r, const rustsecp256k1zkp_v0_5_0_fe *a, int flag) {
+static SECP256K1_INLINE void rustsecp256k1zkp_v0_6_0_fe_cmov(rustsecp256k1zkp_v0_6_0_fe *r, const rustsecp256k1zkp_v0_6_0_fe *a, int flag) {
     uint32_t mask0, mask1;
     VG_CHECK_VERIFY(r->n, sizeof(r->n));
     mask0 = flag + ~((uint32_t)0);
@@ -1118,7 +1133,7 @@ static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_fe_cmov(rustsecp256k1zkp_v0
 #endif
 }
 
-static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_fe_storage_cmov(rustsecp256k1zkp_v0_5_0_fe_storage *r, const rustsecp256k1zkp_v0_5_0_fe_storage *a, int flag) {
+static SECP256K1_INLINE void rustsecp256k1zkp_v0_6_0_fe_storage_cmov(rustsecp256k1zkp_v0_6_0_fe_storage *r, const rustsecp256k1zkp_v0_6_0_fe_storage *a, int flag) {
     uint32_t mask0, mask1;
     VG_CHECK_VERIFY(r->n, sizeof(r->n));
     mask0 = flag + ~((uint32_t)0);
@@ -1133,7 +1148,7 @@ static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_fe_storage_cmov(rustsecp256
     r->n[7] = (r->n[7] & mask0) | (a->n[7] & mask1);
 }
 
-static void rustsecp256k1zkp_v0_5_0_fe_to_storage(rustsecp256k1zkp_v0_5_0_fe_storage *r, const rustsecp256k1zkp_v0_5_0_fe *a) {
+static void rustsecp256k1zkp_v0_6_0_fe_to_storage(rustsecp256k1zkp_v0_6_0_fe_storage *r, const rustsecp256k1zkp_v0_6_0_fe *a) {
 #ifdef VERIFY
     VERIFY_CHECK(a->normalized);
 #endif
@@ -1147,7 +1162,7 @@ static void rustsecp256k1zkp_v0_5_0_fe_to_storage(rustsecp256k1zkp_v0_5_0_fe_sto
     r->n[7] = a->n[8] >> 16 | a->n[9] << 10;
 }
 
-static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_fe_from_storage(rustsecp256k1zkp_v0_5_0_fe *r, const rustsecp256k1zkp_v0_5_0_fe_storage *a) {
+static SECP256K1_INLINE void rustsecp256k1zkp_v0_6_0_fe_from_storage(rustsecp256k1zkp_v0_6_0_fe *r, const rustsecp256k1zkp_v0_6_0_fe_storage *a) {
     r->n[0] = a->n[0] & 0x3FFFFFFUL;
     r->n[1] = a->n[0] >> 26 | ((a->n[1] << 6) & 0x3FFFFFFUL);
     r->n[2] = a->n[1] >> 20 | ((a->n[2] << 12) & 0x3FFFFFFUL);
@@ -1161,7 +1176,96 @@ static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_fe_from_storage(rustsecp256
 #ifdef VERIFY
     r->magnitude = 1;
     r->normalized = 1;
+    rustsecp256k1zkp_v0_6_0_fe_verify(r);
 #endif
 }
 
+static void rustsecp256k1zkp_v0_6_0_fe_from_signed30(rustsecp256k1zkp_v0_6_0_fe *r, const rustsecp256k1zkp_v0_6_0_modinv32_signed30 *a) {
+    const uint32_t M26 = UINT32_MAX >> 6;
+    const uint32_t a0 = a->v[0], a1 = a->v[1], a2 = a->v[2], a3 = a->v[3], a4 = a->v[4],
+                   a5 = a->v[5], a6 = a->v[6], a7 = a->v[7], a8 = a->v[8];
+
+    /* The output from rustsecp256k1zkp_v0_6_0_modinv32{_var} should be normalized to range [0,modulus), and
+     * have limbs in [0,2^30). The modulus is < 2^256, so the top limb must be below 2^(256-30*8).
+     */
+    VERIFY_CHECK(a0 >> 30 == 0);
+    VERIFY_CHECK(a1 >> 30 == 0);
+    VERIFY_CHECK(a2 >> 30 == 0);
+    VERIFY_CHECK(a3 >> 30 == 0);
+    VERIFY_CHECK(a4 >> 30 == 0);
+    VERIFY_CHECK(a5 >> 30 == 0);
+    VERIFY_CHECK(a6 >> 30 == 0);
+    VERIFY_CHECK(a7 >> 30 == 0);
+    VERIFY_CHECK(a8 >> 16 == 0);
+
+    r->n[0] =  a0                   & M26;
+    r->n[1] = (a0 >> 26 | a1 <<  4) & M26;
+    r->n[2] = (a1 >> 22 | a2 <<  8) & M26;
+    r->n[3] = (a2 >> 18 | a3 << 12) & M26;
+    r->n[4] = (a3 >> 14 | a4 << 16) & M26;
+    r->n[5] = (a4 >> 10 | a5 << 20) & M26;
+    r->n[6] = (a5 >>  6 | a6 << 24) & M26;
+    r->n[7] = (a6 >>  2           ) & M26;
+    r->n[8] = (a6 >> 28 | a7 <<  2) & M26;
+    r->n[9] = (a7 >> 24 | a8 <<  6);
+
+#ifdef VERIFY
+    r->magnitude = 1;
+    r->normalized = 1;
+    rustsecp256k1zkp_v0_6_0_fe_verify(r);
+#endif
+}
+
+static void rustsecp256k1zkp_v0_6_0_fe_to_signed30(rustsecp256k1zkp_v0_6_0_modinv32_signed30 *r, const rustsecp256k1zkp_v0_6_0_fe *a) {
+    const uint32_t M30 = UINT32_MAX >> 2;
+    const uint64_t a0 = a->n[0], a1 = a->n[1], a2 = a->n[2], a3 = a->n[3], a4 = a->n[4],
+                   a5 = a->n[5], a6 = a->n[6], a7 = a->n[7], a8 = a->n[8], a9 = a->n[9];
+
+#ifdef VERIFY
+    VERIFY_CHECK(a->normalized);
+#endif
+
+    r->v[0] = (a0       | a1 << 26) & M30;
+    r->v[1] = (a1 >>  4 | a2 << 22) & M30;
+    r->v[2] = (a2 >>  8 | a3 << 18) & M30;
+    r->v[3] = (a3 >> 12 | a4 << 14) & M30;
+    r->v[4] = (a4 >> 16 | a5 << 10) & M30;
+    r->v[5] = (a5 >> 20 | a6 <<  6) & M30;
+    r->v[6] = (a6 >> 24 | a7 <<  2
+                        | a8 << 28) & M30;
+    r->v[7] = (a8 >>  2 | a9 << 24) & M30;
+    r->v[8] =  a9 >>  6;
+}
+
+static const rustsecp256k1zkp_v0_6_0_modinv32_modinfo rustsecp256k1zkp_v0_6_0_const_modinfo_fe = {
+    {{-0x3D1, -4, 0, 0, 0, 0, 0, 0, 65536}},
+    0x2DDACACFL
+};
+
+static void rustsecp256k1zkp_v0_6_0_fe_inv(rustsecp256k1zkp_v0_6_0_fe *r, const rustsecp256k1zkp_v0_6_0_fe *x) {
+    rustsecp256k1zkp_v0_6_0_fe tmp;
+    rustsecp256k1zkp_v0_6_0_modinv32_signed30 s;
+
+    tmp = *x;
+    rustsecp256k1zkp_v0_6_0_fe_normalize(&tmp);
+    rustsecp256k1zkp_v0_6_0_fe_to_signed30(&s, &tmp);
+    rustsecp256k1zkp_v0_6_0_modinv32(&s, &rustsecp256k1zkp_v0_6_0_const_modinfo_fe);
+    rustsecp256k1zkp_v0_6_0_fe_from_signed30(r, &s);
+
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_fe_normalizes_to_zero(r) == rustsecp256k1zkp_v0_6_0_fe_normalizes_to_zero(&tmp));
+}
+
+static void rustsecp256k1zkp_v0_6_0_fe_inv_var(rustsecp256k1zkp_v0_6_0_fe *r, const rustsecp256k1zkp_v0_6_0_fe *x) {
+    rustsecp256k1zkp_v0_6_0_fe tmp;
+    rustsecp256k1zkp_v0_6_0_modinv32_signed30 s;
+
+    tmp = *x;
+    rustsecp256k1zkp_v0_6_0_fe_normalize_var(&tmp);
+    rustsecp256k1zkp_v0_6_0_fe_to_signed30(&s, &tmp);
+    rustsecp256k1zkp_v0_6_0_modinv32_var(&s, &rustsecp256k1zkp_v0_6_0_const_modinfo_fe);
+    rustsecp256k1zkp_v0_6_0_fe_from_signed30(r, &s);
+
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_fe_normalizes_to_zero(r) == rustsecp256k1zkp_v0_6_0_fe_normalizes_to_zero(&tmp));
+}
+
 #endif /* SECP256K1_FIELD_REPR_IMPL_H */
