@@ -21,64 +21,70 @@ void test_generator_api(void) {
     unsigned char key[32];
     unsigned char blind[32];
     unsigned char sergen[33];
-    rustsecp256k1zkp_v0_5_0_context *none = rustsecp256k1zkp_v0_5_0_context_create(SECP256K1_CONTEXT_NONE);
-    rustsecp256k1zkp_v0_5_0_context *sign = rustsecp256k1zkp_v0_5_0_context_create(SECP256K1_CONTEXT_SIGN);
-    rustsecp256k1zkp_v0_5_0_context *vrfy = rustsecp256k1zkp_v0_5_0_context_create(SECP256K1_CONTEXT_VERIFY);
-    rustsecp256k1zkp_v0_5_0_generator gen;
+    rustsecp256k1zkp_v0_6_0_context *none = rustsecp256k1zkp_v0_6_0_context_create(SECP256K1_CONTEXT_NONE);
+    rustsecp256k1zkp_v0_6_0_context *sign = rustsecp256k1zkp_v0_6_0_context_create(SECP256K1_CONTEXT_SIGN);
+    rustsecp256k1zkp_v0_6_0_context *vrfy = rustsecp256k1zkp_v0_6_0_context_create(SECP256K1_CONTEXT_VERIFY);
+    rustsecp256k1zkp_v0_6_0_context *sttc = rustsecp256k1zkp_v0_6_0_context_clone(rustsecp256k1zkp_v0_6_0_context_no_precomp);
+    rustsecp256k1zkp_v0_6_0_generator gen;
     int32_t ecount = 0;
 
-    rustsecp256k1zkp_v0_5_0_context_set_error_callback(none, counting_illegal_callback_fn, &ecount);
-    rustsecp256k1zkp_v0_5_0_context_set_error_callback(sign, counting_illegal_callback_fn, &ecount);
-    rustsecp256k1zkp_v0_5_0_context_set_error_callback(vrfy, counting_illegal_callback_fn, &ecount);
-    rustsecp256k1zkp_v0_5_0_context_set_illegal_callback(none, counting_illegal_callback_fn, &ecount);
-    rustsecp256k1zkp_v0_5_0_context_set_illegal_callback(sign, counting_illegal_callback_fn, &ecount);
-    rustsecp256k1zkp_v0_5_0_context_set_illegal_callback(vrfy, counting_illegal_callback_fn, &ecount);
-    rustsecp256k1zkp_v0_5_0_testrand256(key);
-    rustsecp256k1zkp_v0_5_0_testrand256(blind);
+    rustsecp256k1zkp_v0_6_0_context_set_error_callback(none, counting_illegal_callback_fn, &ecount);
+    rustsecp256k1zkp_v0_6_0_context_set_error_callback(sign, counting_illegal_callback_fn, &ecount);
+    rustsecp256k1zkp_v0_6_0_context_set_error_callback(vrfy, counting_illegal_callback_fn, &ecount);
+    rustsecp256k1zkp_v0_6_0_context_set_error_callback(sttc, counting_illegal_callback_fn, &ecount);
+    rustsecp256k1zkp_v0_6_0_context_set_illegal_callback(none, counting_illegal_callback_fn, &ecount);
+    rustsecp256k1zkp_v0_6_0_context_set_illegal_callback(sign, counting_illegal_callback_fn, &ecount);
+    rustsecp256k1zkp_v0_6_0_context_set_illegal_callback(vrfy, counting_illegal_callback_fn, &ecount);
+    rustsecp256k1zkp_v0_6_0_context_set_illegal_callback(sttc, counting_illegal_callback_fn, &ecount);
+    rustsecp256k1zkp_v0_6_0_testrand256(key);
+    rustsecp256k1zkp_v0_6_0_testrand256(blind);
 
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_generate(none, &gen, key) == 1);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_generate(none, &gen, key) == 1);
     CHECK(ecount == 0);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_generate(none, NULL, key) == 0);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_generate(none, NULL, key) == 0);
     CHECK(ecount == 1);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_generate(none, &gen, NULL) == 0);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_generate(none, &gen, NULL) == 0);
     CHECK(ecount == 2);
 
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_generate_blinded(sign, &gen, key, blind) == 1);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_generate_blinded(sign, &gen, key, blind) == 1);
     CHECK(ecount == 2);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_generate_blinded(vrfy, &gen, key, blind) == 0);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_generate_blinded(vrfy, &gen, key, blind) == 1);
+    CHECK(ecount == 2);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_generate_blinded(none, &gen, key, blind) == 1);
+    CHECK(ecount == 2);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_generate_blinded(sttc, &gen, key, blind) == 0);
     CHECK(ecount == 3);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_generate_blinded(none, &gen, key, blind) == 0);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_generate_blinded(vrfy, NULL, key, blind) == 0);
     CHECK(ecount == 4);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_generate_blinded(vrfy, NULL, key, blind) == 0);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_generate_blinded(vrfy, &gen, NULL, blind) == 0);
     CHECK(ecount == 5);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_generate_blinded(vrfy, &gen, NULL, blind) == 0);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_generate_blinded(vrfy, &gen, key, NULL) == 0);
     CHECK(ecount == 6);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_generate_blinded(vrfy, &gen, key, NULL) == 0);
-    CHECK(ecount == 7);
 
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_serialize(none, sergen, &gen) == 1);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_serialize(none, sergen, &gen) == 1);
+    CHECK(ecount == 6);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_serialize(none, NULL, &gen) == 0);
     CHECK(ecount == 7);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_serialize(none, NULL, &gen) == 0);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_serialize(none, sergen, NULL) == 0);
     CHECK(ecount == 8);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_serialize(none, sergen, NULL) == 0);
-    CHECK(ecount == 9);
 
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_serialize(none, sergen, &gen) == 1);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_parse(none, &gen, sergen) == 1);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_serialize(none, sergen, &gen) == 1);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_parse(none, &gen, sergen) == 1);
+    CHECK(ecount == 8);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_parse(none, NULL, sergen) == 0);
     CHECK(ecount == 9);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_parse(none, NULL, sergen) == 0);
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_parse(none, &gen, NULL) == 0);
     CHECK(ecount == 10);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_parse(none, &gen, NULL) == 0);
-    CHECK(ecount == 11);
 
-    rustsecp256k1zkp_v0_5_0_context_destroy(none);
-    rustsecp256k1zkp_v0_5_0_context_destroy(sign);
-    rustsecp256k1zkp_v0_5_0_context_destroy(vrfy);
+    rustsecp256k1zkp_v0_6_0_context_destroy(none);
+    rustsecp256k1zkp_v0_6_0_context_destroy(sign);
+    rustsecp256k1zkp_v0_6_0_context_destroy(vrfy);
+    rustsecp256k1zkp_v0_6_0_context_destroy(sttc);
 }
 
 void test_shallue_van_de_woestijne(void) {
     /* Matches with the output of the shallue_van_de_woestijne.sage SAGE program */
-    static const rustsecp256k1zkp_v0_5_0_ge_storage results[32] = {
+    static const rustsecp256k1zkp_v0_6_0_ge_storage results[32] = {
         SECP256K1_GE_STORAGE_CONST(0xedd1fd3e, 0x327ce90c, 0xc7a35426, 0x14289aee, 0x9682003e, 0x9cf7dcc9, 0xcf2ca974, 0x3be5aa0c, 0x0225f529, 0xee75acaf, 0xccfc4560, 0x26c5e46b, 0xf80237a3, 0x3924655a, 0x16f90e88, 0x085ed52a),
         SECP256K1_GE_STORAGE_CONST(0xedd1fd3e, 0x327ce90c, 0xc7a35426, 0x14289aee, 0x9682003e, 0x9cf7dcc9, 0xcf2ca974, 0x3be5aa0c, 0xfdda0ad6, 0x118a5350, 0x3303ba9f, 0xd93a1b94, 0x07fdc85c, 0xc6db9aa5, 0xe906f176, 0xf7a12705),
         SECP256K1_GE_STORAGE_CONST(0x2c5cdc9c, 0x338152fa, 0x85de92cb, 0x1bee9907, 0x765a922e, 0x4f037cce, 0x14ecdbf2, 0x2f78fe15, 0x56716069, 0x6818286b, 0x72f01a3e, 0x5e8caca7, 0x36249160, 0xc7ded69d, 0xd51913c3, 0x03a2fa97),
@@ -113,28 +119,28 @@ void test_shallue_van_de_woestijne(void) {
         SECP256K1_GE_STORAGE_CONST(0xf75763bc, 0x2907e79b, 0x125e33c3, 0x9a027f48, 0x0f8c6409, 0x2153432f, 0x967bc2b1, 0x1d1f5cf0, 0x4b571239, 0xc9c6e4c6, 0x643de63f, 0xc2fcced7, 0x2432b9c1, 0x2daf9c6b, 0xf9e47859, 0x61aef9fa),
     };
 
-    rustsecp256k1zkp_v0_5_0_ge ge;
-    rustsecp256k1zkp_v0_5_0_fe fe;
-    rustsecp256k1zkp_v0_5_0_ge_storage ges;
+    rustsecp256k1zkp_v0_6_0_ge ge;
+    rustsecp256k1zkp_v0_6_0_fe fe;
+    rustsecp256k1zkp_v0_6_0_ge_storage ges;
     int i, s;
     for (i = 1; i <= 16; i++) {
-        rustsecp256k1zkp_v0_5_0_fe_set_int(&fe, i);
+        rustsecp256k1zkp_v0_6_0_fe_set_int(&fe, i);
 
         for (s = 0; s < 2; s++) {
             if (s) {
-                rustsecp256k1zkp_v0_5_0_fe_negate(&fe, &fe, 1);
-                rustsecp256k1zkp_v0_5_0_fe_normalize(&fe);
+                rustsecp256k1zkp_v0_6_0_fe_negate(&fe, &fe, 1);
+                rustsecp256k1zkp_v0_6_0_fe_normalize(&fe);
             }
             shallue_van_de_woestijne(&ge, &fe);
-            rustsecp256k1zkp_v0_5_0_ge_to_storage(&ges, &ge);
+            rustsecp256k1zkp_v0_6_0_ge_to_storage(&ges, &ge);
 
-            CHECK(memcmp(&ges, &results[i * 2 + s - 2], sizeof(rustsecp256k1zkp_v0_5_0_ge_storage)) == 0);
+            CHECK(memcmp(&ges, &results[i * 2 + s - 2], sizeof(rustsecp256k1zkp_v0_6_0_ge_storage)) == 0);
         }
     }
 }
 
 void test_generator_generate(void) {
-    static const rustsecp256k1zkp_v0_5_0_ge_storage results[32] = {
+    static const rustsecp256k1zkp_v0_6_0_ge_storage results[32] = {
         SECP256K1_GE_STORAGE_CONST(0x806cd8ed, 0xd6c153e3, 0x4aa9b9a0, 0x8755c4be, 0x4718b1ef, 0xb26cb93f, 0xfdd99e1b, 0x21f2af8e, 0xc7062208, 0xcc649a03, 0x1bdc1a33, 0x9d01f115, 0x4bcd0dca, 0xfe0b875d, 0x62f35f73, 0x28673006),
         SECP256K1_GE_STORAGE_CONST(0xd91b15ec, 0x47a811f4, 0xaa189561, 0xd13f5c4d, 0x4e81f10d, 0xc7dc551f, 0x4fea9b84, 0x610314c4, 0x9b0ada1e, 0xb38efd67, 0x8bff0b6c, 0x7d7315f7, 0xb49b8cc5, 0xa679fad4, 0xc94f9dc6, 0x9da66382),
         SECP256K1_GE_STORAGE_CONST(0x11c00de6, 0xf885035e, 0x76051430, 0xa3c38b2a, 0x5f86ab8c, 0xf66dae58, 0x04ea7307, 0x348b19bf, 0xe0858ae7, 0x61dcb1ba, 0xff247e37, 0xd38fcd88, 0xf3bd7911, 0xaa4ed6e0, 0x28d792dd, 0x3ee1ac09),
@@ -168,34 +174,34 @@ void test_generator_generate(void) {
         SECP256K1_GE_STORAGE_CONST(0x150df593, 0x5b6956a0, 0x0cfed843, 0xb9d6ffce, 0x4f790022, 0xea18730f, 0xc495111d, 0x91568e55, 0x6700a2ca, 0x9ff4ed32, 0xc1697312, 0x4eb51ce3, 0x5656344b, 0x65a1e3d5, 0xd6c1f7ce, 0x29233f82),
         SECP256K1_GE_STORAGE_CONST(0x38e02eaf, 0x2c8774fd, 0x58b8b373, 0x732457f1, 0x16dbe53b, 0xea5683d9, 0xada20dd7, 0x14ce20a6, 0x6ac5362e, 0xbb425416, 0x8250f43f, 0xa4ee2b63, 0x0406324f, 0x1c876d60, 0xebe5be2c, 0x6eb1515b),
     };
-    rustsecp256k1zkp_v0_5_0_generator gen;
-    rustsecp256k1zkp_v0_5_0_ge ge;
-    rustsecp256k1zkp_v0_5_0_ge_storage ges;
+    rustsecp256k1zkp_v0_6_0_generator gen;
+    rustsecp256k1zkp_v0_6_0_ge ge;
+    rustsecp256k1zkp_v0_6_0_ge_storage ges;
     int i;
     unsigned char v[32];
     unsigned char s[32] = {0};
-    rustsecp256k1zkp_v0_5_0_scalar sc;
-    rustsecp256k1zkp_v0_5_0_scalar_set_b32(&sc, s, NULL);
+    rustsecp256k1zkp_v0_6_0_scalar sc;
+    rustsecp256k1zkp_v0_6_0_scalar_set_b32(&sc, s, NULL);
     for (i = 1; i <= 32; i++) {
         memset(v, 0, 31);
         v[31] = i;
-        CHECK(rustsecp256k1zkp_v0_5_0_generator_generate_blinded(ctx, &gen, v, s));
-        rustsecp256k1zkp_v0_5_0_generator_load(&ge, &gen);
-        rustsecp256k1zkp_v0_5_0_ge_to_storage(&ges, &ge);
-        CHECK(memcmp(&ges, &results[i - 1], sizeof(rustsecp256k1zkp_v0_5_0_ge_storage)) == 0);
-        CHECK(rustsecp256k1zkp_v0_5_0_generator_generate(ctx, &gen, v));
-        rustsecp256k1zkp_v0_5_0_generator_load(&ge, &gen);
-        rustsecp256k1zkp_v0_5_0_ge_to_storage(&ges, &ge);
-        CHECK(memcmp(&ges, &results[i - 1], sizeof(rustsecp256k1zkp_v0_5_0_ge_storage)) == 0);
+        CHECK(rustsecp256k1zkp_v0_6_0_generator_generate_blinded(ctx, &gen, v, s));
+        rustsecp256k1zkp_v0_6_0_generator_load(&ge, &gen);
+        rustsecp256k1zkp_v0_6_0_ge_to_storage(&ges, &ge);
+        CHECK(memcmp(&ges, &results[i - 1], sizeof(rustsecp256k1zkp_v0_6_0_ge_storage)) == 0);
+        CHECK(rustsecp256k1zkp_v0_6_0_generator_generate(ctx, &gen, v));
+        rustsecp256k1zkp_v0_6_0_generator_load(&ge, &gen);
+        rustsecp256k1zkp_v0_6_0_ge_to_storage(&ges, &ge);
+        CHECK(memcmp(&ges, &results[i - 1], sizeof(rustsecp256k1zkp_v0_6_0_ge_storage)) == 0);
     }
 
     /* There is no range restriction on the value, but the blinder must be a
      * valid scalar. Check that an invalid blinder causes the call to fail
      * but not crash. */
     memset(v, 0xff, 32);
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_generate(ctx, &gen, v));
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_generate(ctx, &gen, v));
     memset(s, 0xff, 32);
-    CHECK(!rustsecp256k1zkp_v0_5_0_generator_generate_blinded(ctx, &gen, v, s));
+    CHECK(!rustsecp256k1zkp_v0_6_0_generator_generate_blinded(ctx, &gen, v, s));
 }
 
 void test_generator_fixed_vector(void) {
@@ -205,16 +211,16 @@ void test_generator_fixed_vector(void) {
         0x5c, 0x77, 0x8e, 0x4b, 0x8c, 0xef, 0x3c, 0xa7, 0xab, 0xac, 0x09, 0xb9, 0x5c, 0x70, 0x9e, 0xe5
     };
     unsigned char result[33];
-    rustsecp256k1zkp_v0_5_0_generator parse;
+    rustsecp256k1zkp_v0_6_0_generator parse;
 
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_parse(ctx, &parse, two_g));
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_serialize(ctx, result, &parse));
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_parse(ctx, &parse, two_g));
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_serialize(ctx, result, &parse));
     CHECK(memcmp(two_g, result, 33) == 0);
 
     result[0] = 0x0a;
-    CHECK(rustsecp256k1zkp_v0_5_0_generator_parse(ctx, &parse, result));
+    CHECK(rustsecp256k1zkp_v0_6_0_generator_parse(ctx, &parse, result));
     result[0] = 0x08;
-    CHECK(!rustsecp256k1zkp_v0_5_0_generator_parse(ctx, &parse, result));
+    CHECK(!rustsecp256k1zkp_v0_6_0_generator_parse(ctx, &parse, result));
 }
 
 void run_generator_tests(void) {
