@@ -12,10 +12,9 @@
 #include "ecmult.h"
 #include "ecmult_gen.h"
 
-static int rustsecp256k1zkp_v0_5_0_rangeproof_verify_impl(const rustsecp256k1zkp_v0_5_0_ecmult_context* ecmult_ctx,
- const rustsecp256k1zkp_v0_5_0_ecmult_gen_context* ecmult_gen_ctx,
+static int rustsecp256k1zkp_v0_6_0_rangeproof_verify_impl(const rustsecp256k1zkp_v0_6_0_ecmult_gen_context* ecmult_gen_ctx,
  unsigned char *blindout, uint64_t *value_out, unsigned char *message_out, size_t *outlen, const unsigned char *nonce,
- uint64_t *min_value, uint64_t *max_value, const rustsecp256k1zkp_v0_5_0_ge *commit, const unsigned char *proof, size_t plen,
- const unsigned char *extra_commit, size_t extra_commit_len, const rustsecp256k1zkp_v0_5_0_ge* genp);
+ uint64_t *min_value, uint64_t *max_value, const rustsecp256k1zkp_v0_6_0_ge *commit, const unsigned char *proof, size_t plen,
+ const unsigned char *extra_commit, size_t extra_commit_len, const rustsecp256k1zkp_v0_6_0_ge* genp);
 
 #endif
