@@ -23,7 +23,7 @@
 #define SECP256K1_SURJECTIONPROOF_MAX_USED_INPUTS 16
 #endif
 
-static size_t rustsecp256k1zkp_v0_5_0_count_bits_set(const unsigned char* data, size_t count) {
+static size_t rustsecp256k1zkp_v0_6_0_count_bits_set(const unsigned char* data, size_t count) {
     size_t ret = 0;
     size_t i;
     for (i = 0; i < count; i++) {
@@ -46,7 +46,7 @@ static size_t rustsecp256k1zkp_v0_5_0_count_bits_set(const unsigned char* data,
 #ifdef USE_REDUCED_SURJECTION_PROOF_SIZE
 static
 #endif
-int rustsecp256k1zkp_v0_5_0_surjectionproof_parse(const rustsecp256k1zkp_v0_5_0_context* ctx, rustsecp256k1zkp_v0_5_0_surjectionproof *proof, const unsigned char *input, size_t inputlen) {
+int rustsecp256k1zkp_v0_6_0_surjectionproof_parse(const rustsecp256k1zkp_v0_6_0_context* ctx, rustsecp256k1zkp_v0_6_0_surjectionproof *proof, const unsigned char *input, size_t inputlen) {
     size_t n_inputs;
     size_t signature_len;
 
@@ -75,7 +75,7 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_parse(const rustsecp256k1zkp_v0_5_0_
         }
     }
 
-    signature_len = 32 * (1 + rustsecp256k1zkp_v0_5_0_count_bits_set(&input[2], (n_inputs + 7) / 8));
+    signature_len = 32 * (1 + rustsecp256k1zkp_v0_6_0_count_bits_set(&input[2], (n_inputs + 7) / 8));
     if (inputlen != 2 + (n_inputs + 7) / 8 + signature_len) {
         return 0;
     }
@@ -86,7 +86,7 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_parse(const rustsecp256k1zkp_v0_5_0_
     return 1;
 }
 
-int rustsecp256k1zkp_v0_5_0_surjectionproof_serialize(const rustsecp256k1zkp_v0_5_0_context* ctx, unsigned char *output, size_t *outputlen, const rustsecp256k1zkp_v0_5_0_surjectionproof *proof) {
+int rustsecp256k1zkp_v0_6_0_surjectionproof_serialize(const rustsecp256k1zkp_v0_6_0_context* ctx, unsigned char *output, size_t *outputlen, const rustsecp256k1zkp_v0_6_0_surjectionproof *proof) {
     size_t signature_len;
     size_t serialized_len;
 
@@ -96,7 +96,7 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_serialize(const rustsecp256k1zkp_v0_
     ARG_CHECK(proof != NULL);
     (void) ctx;
 
-    signature_len = 32 * (1 + rustsecp256k1zkp_v0_5_0_count_bits_set(proof->used_inputs, (proof->n_inputs + 7) / 8));
+    signature_len = 32 * (1 + rustsecp256k1zkp_v0_6_0_count_bits_set(proof->used_inputs, (proof->n_inputs + 7) / 8));
     serialized_len = 2 + (proof->n_inputs + 7) / 8 + signature_len;
     if (*outputlen < serialized_len) {
         return 0;
@@ -111,37 +111,37 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_serialize(const rustsecp256k1zkp_v0_
     return 1;
 }
 
-size_t rustsecp256k1zkp_v0_5_0_surjectionproof_n_total_inputs(const rustsecp256k1zkp_v0_5_0_context* ctx, const rustsecp256k1zkp_v0_5_0_surjectionproof* proof) {
+size_t rustsecp256k1zkp_v0_6_0_surjectionproof_n_total_inputs(const rustsecp256k1zkp_v0_6_0_context* ctx, const rustsecp256k1zkp_v0_6_0_surjectionproof* proof) {
     VERIFY_CHECK(ctx != NULL);
     ARG_CHECK(proof != NULL);
     (void) ctx;
     return proof->n_inputs;
 }
 
-size_t rustsecp256k1zkp_v0_5_0_surjectionproof_n_used_inputs(const rustsecp256k1zkp_v0_5_0_context* ctx, const rustsecp256k1zkp_v0_5_0_surjectionproof* proof) {
+size_t rustsecp256k1zkp_v0_6_0_surjectionproof_n_used_inputs(const rustsecp256k1zkp_v0_6_0_context* ctx, const rustsecp256k1zkp_v0_6_0_surjectionproof* proof) {
     VERIFY_CHECK(ctx != NULL);
     ARG_CHECK(proof != NULL);
     (void) ctx;
-    return rustsecp256k1zkp_v0_5_0_count_bits_set(proof->used_inputs, (proof->n_inputs + 7) / 8);
+    return rustsecp256k1zkp_v0_6_0_count_bits_set(proof->used_inputs, (proof->n_inputs + 7) / 8);
 }
 
-size_t rustsecp256k1zkp_v0_5_0_surjectionproof_serialized_size(const rustsecp256k1zkp_v0_5_0_context* ctx, const rustsecp256k1zkp_v0_5_0_surjectionproof* proof) {
+size_t rustsecp256k1zkp_v0_6_0_surjectionproof_serialized_size(const rustsecp256k1zkp_v0_6_0_context* ctx, const rustsecp256k1zkp_v0_6_0_surjectionproof* proof) {
     VERIFY_CHECK(ctx != NULL);
     ARG_CHECK(proof != NULL);
-    return 2 + (proof->n_inputs + 7) / 8 + 32 * (1 + rustsecp256k1zkp_v0_5_0_surjectionproof_n_used_inputs(ctx, proof));
+    return 2 + (proof->n_inputs + 7) / 8 + 32 * (1 + rustsecp256k1zkp_v0_6_0_surjectionproof_n_used_inputs(ctx, proof));
 }
 
 typedef struct {
     unsigned char state[32];
     size_t state_i;
-} rustsecp256k1zkp_v0_5_0_surjectionproof_csprng;
+} rustsecp256k1zkp_v0_6_0_surjectionproof_csprng;
 
-static void rustsecp256k1zkp_v0_5_0_surjectionproof_csprng_init(rustsecp256k1zkp_v0_5_0_surjectionproof_csprng *csprng, const unsigned char* state) {
+static void rustsecp256k1zkp_v0_6_0_surjectionproof_csprng_init(rustsecp256k1zkp_v0_6_0_surjectionproof_csprng *csprng, const unsigned char* state) {
     memcpy(csprng->state, state, 32);
     csprng->state_i = 0;
 }
 
-static size_t rustsecp256k1zkp_v0_5_0_surjectionproof_csprng_next(rustsecp256k1zkp_v0_5_0_surjectionproof_csprng *csprng, size_t rand_max) {
+static size_t rustsecp256k1zkp_v0_6_0_surjectionproof_csprng_next(rustsecp256k1zkp_v0_6_0_surjectionproof_csprng *csprng, size_t rand_max) {
     /* The number of random bytes to read for each random sample */
     const size_t increment = rand_max > 256 ? 2 : 1;
     /* The maximum value expressable by the number of random bytes we read */
@@ -152,10 +152,10 @@ static size_t rustsecp256k1zkp_v0_5_0_surjectionproof_csprng_next(rustsecp256k1z
     while (1) {
         size_t val;
         if (csprng->state_i + increment >= 32) {
-            rustsecp256k1zkp_v0_5_0_sha256 sha;
-            rustsecp256k1zkp_v0_5_0_sha256_initialize(&sha);
-            rustsecp256k1zkp_v0_5_0_sha256_write(&sha, csprng->state, 32);
-            rustsecp256k1zkp_v0_5_0_sha256_finalize(&sha, csprng->state);
+            rustsecp256k1zkp_v0_6_0_sha256 sha;
+            rustsecp256k1zkp_v0_6_0_sha256_initialize(&sha);
+            rustsecp256k1zkp_v0_6_0_sha256_write(&sha, csprng->state, 32);
+            rustsecp256k1zkp_v0_6_0_sha256_finalize(&sha, csprng->state);
             csprng->state_i = 0;
         }
         val = csprng->state[csprng->state_i];
@@ -171,8 +171,8 @@ static size_t rustsecp256k1zkp_v0_5_0_surjectionproof_csprng_next(rustsecp256k1z
     }
 }
 
-int rustsecp256k1zkp_v0_5_0_surjectionproof_initialize(const rustsecp256k1zkp_v0_5_0_context* ctx, rustsecp256k1zkp_v0_5_0_surjectionproof* proof, size_t *input_index, const rustsecp256k1zkp_v0_5_0_fixed_asset_tag* fixed_input_tags, const size_t n_input_tags, const size_t n_input_tags_to_use, const rustsecp256k1zkp_v0_5_0_fixed_asset_tag* fixed_output_tag, const size_t n_max_iterations, const unsigned char *random_seed32) {
-    rustsecp256k1zkp_v0_5_0_surjectionproof_csprng csprng;
+int rustsecp256k1zkp_v0_6_0_surjectionproof_initialize(const rustsecp256k1zkp_v0_6_0_context* ctx, rustsecp256k1zkp_v0_6_0_surjectionproof* proof, size_t *input_index, const rustsecp256k1zkp_v0_6_0_fixed_asset_tag* fixed_input_tags, const size_t n_input_tags, const size_t n_input_tags_to_use, const rustsecp256k1zkp_v0_6_0_fixed_asset_tag* fixed_output_tag, const size_t n_max_iterations, const unsigned char *random_seed32) {
+    rustsecp256k1zkp_v0_6_0_surjectionproof_csprng csprng;
     size_t n_iterations = 0;
 
     VERIFY_CHECK(ctx != NULL);
@@ -186,7 +186,7 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_initialize(const rustsecp256k1zkp_v0
     ARG_CHECK(n_input_tags_to_use <= n_input_tags);
     (void) ctx;
 
-    rustsecp256k1zkp_v0_5_0_surjectionproof_csprng_init(&csprng, random_seed32);
+    rustsecp256k1zkp_v0_6_0_surjectionproof_csprng_init(&csprng, random_seed32);
     memset(proof->data, 0, sizeof(proof->data));
     proof->n_inputs = n_input_tags;
 
@@ -199,7 +199,7 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_initialize(const rustsecp256k1zkp_v0
         for (i = 0; i < n_input_tags_to_use; i++) {
             while (1) {
                 size_t next_input_index;
-                next_input_index = rustsecp256k1zkp_v0_5_0_surjectionproof_csprng_next(&csprng, n_input_tags);
+                next_input_index = rustsecp256k1zkp_v0_6_0_surjectionproof_csprng_next(&csprng, n_input_tags);
                 if (memcmp(&fixed_input_tags[next_input_index], fixed_output_tag, sizeof(*fixed_output_tag)) == 0) {
                     *input_index = next_input_index;
                     has_output_tag = 1;
@@ -229,10 +229,10 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_initialize(const rustsecp256k1zkp_v0
     }
 }
 
-int rustsecp256k1zkp_v0_5_0_surjectionproof_generate(const rustsecp256k1zkp_v0_5_0_context* ctx, rustsecp256k1zkp_v0_5_0_surjectionproof* proof, const rustsecp256k1zkp_v0_5_0_generator* ephemeral_input_tags, size_t n_ephemeral_input_tags, const rustsecp256k1zkp_v0_5_0_generator* ephemeral_output_tag, size_t input_index, const unsigned char *input_blinding_key, const unsigned char *output_blinding_key) {
-    rustsecp256k1zkp_v0_5_0_scalar blinding_key;
-    rustsecp256k1zkp_v0_5_0_scalar tmps;
-    rustsecp256k1zkp_v0_5_0_scalar nonce;
+int rustsecp256k1zkp_v0_6_0_surjectionproof_generate(const rustsecp256k1zkp_v0_6_0_context* ctx, rustsecp256k1zkp_v0_6_0_surjectionproof* proof, const rustsecp256k1zkp_v0_6_0_generator* ephemeral_input_tags, size_t n_ephemeral_input_tags, const rustsecp256k1zkp_v0_6_0_generator* ephemeral_output_tag, size_t input_index, const unsigned char *input_blinding_key, const unsigned char *output_blinding_key) {
+    rustsecp256k1zkp_v0_6_0_scalar blinding_key;
+    rustsecp256k1zkp_v0_6_0_scalar tmps;
+    rustsecp256k1zkp_v0_6_0_scalar nonce;
     int overflow = 0;
     size_t rsizes[1];    /* array needed for borromean sig API */
     size_t indices[1];   /* array needed for borromean sig API */
@@ -240,13 +240,12 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_generate(const rustsecp256k1zkp_v0_5
     size_t n_total_pubkeys;
     size_t n_used_pubkeys;
     size_t ring_input_index = 0;
-    rustsecp256k1zkp_v0_5_0_gej ring_pubkeys[SECP256K1_SURJECTIONPROOF_MAX_USED_INPUTS];
-    rustsecp256k1zkp_v0_5_0_scalar borromean_s[SECP256K1_SURJECTIONPROOF_MAX_USED_INPUTS];
+    rustsecp256k1zkp_v0_6_0_gej ring_pubkeys[SECP256K1_SURJECTIONPROOF_MAX_USED_INPUTS];
+    rustsecp256k1zkp_v0_6_0_scalar borromean_s[SECP256K1_SURJECTIONPROOF_MAX_USED_INPUTS];
     unsigned char msg32[32];
 
     VERIFY_CHECK(ctx != NULL);
-    ARG_CHECK(rustsecp256k1zkp_v0_5_0_ecmult_context_is_built(&ctx->ecmult_ctx));
-    ARG_CHECK(rustsecp256k1zkp_v0_5_0_ecmult_gen_context_is_built(&ctx->ecmult_gen_ctx));
+    ARG_CHECK(rustsecp256k1zkp_v0_6_0_ecmult_gen_context_is_built(&ctx->ecmult_gen_ctx));
     ARG_CHECK(proof != NULL);
     ARG_CHECK(ephemeral_input_tags != NULL);
     ARG_CHECK(ephemeral_output_tag != NULL);
@@ -257,50 +256,50 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_generate(const rustsecp256k1zkp_v0_5
 #endif
 
     /* Compute secret key */
-    rustsecp256k1zkp_v0_5_0_scalar_set_b32(&tmps, input_blinding_key, &overflow);
+    rustsecp256k1zkp_v0_6_0_scalar_set_b32(&tmps, input_blinding_key, &overflow);
     if (overflow) {
         return 0;
     }
-    rustsecp256k1zkp_v0_5_0_scalar_set_b32(&blinding_key, output_blinding_key, &overflow);
+    rustsecp256k1zkp_v0_6_0_scalar_set_b32(&blinding_key, output_blinding_key, &overflow);
     if (overflow) {
         return 0;
     }
     /* The only time the input may equal the output is if neither one was blinded in the first place,
      * i.e. both blinding keys are zero. Otherwise this is a privacy leak. */
-    if (rustsecp256k1zkp_v0_5_0_scalar_eq(&tmps, &blinding_key) && !rustsecp256k1zkp_v0_5_0_scalar_is_zero(&blinding_key)) {
+    if (rustsecp256k1zkp_v0_6_0_scalar_eq(&tmps, &blinding_key) && !rustsecp256k1zkp_v0_6_0_scalar_is_zero(&blinding_key)) {
         return 0;
     }
-    rustsecp256k1zkp_v0_5_0_scalar_negate(&tmps, &tmps);
-    rustsecp256k1zkp_v0_5_0_scalar_add(&blinding_key, &blinding_key, &tmps);
+    rustsecp256k1zkp_v0_6_0_scalar_negate(&tmps, &tmps);
+    rustsecp256k1zkp_v0_6_0_scalar_add(&blinding_key, &blinding_key, &tmps);
 
     /* Compute public keys */
-    n_total_pubkeys = rustsecp256k1zkp_v0_5_0_surjectionproof_n_total_inputs(ctx, proof);
-    n_used_pubkeys = rustsecp256k1zkp_v0_5_0_surjectionproof_n_used_inputs(ctx, proof);
+    n_total_pubkeys = rustsecp256k1zkp_v0_6_0_surjectionproof_n_total_inputs(ctx, proof);
+    n_used_pubkeys = rustsecp256k1zkp_v0_6_0_surjectionproof_n_used_inputs(ctx, proof);
     if (n_used_pubkeys > n_total_pubkeys || n_total_pubkeys != n_ephemeral_input_tags) {
         return 0;
     }
 
-    if (rustsecp256k1zkp_v0_5_0_surjection_compute_public_keys(ring_pubkeys, n_used_pubkeys, ephemeral_input_tags, n_total_pubkeys, proof->used_inputs, ephemeral_output_tag, input_index, &ring_input_index) == 0) {
+    if (rustsecp256k1zkp_v0_6_0_surjection_compute_public_keys(ring_pubkeys, n_used_pubkeys, ephemeral_input_tags, n_total_pubkeys, proof->used_inputs, ephemeral_output_tag, input_index, &ring_input_index) == 0) {
         return 0;
     }
 
     /* Produce signature */
     rsizes[0] = (int) n_used_pubkeys;
     indices[0] = (int) ring_input_index;
-    rustsecp256k1zkp_v0_5_0_surjection_genmessage(msg32, ephemeral_input_tags, n_total_pubkeys, ephemeral_output_tag);
-    if (rustsecp256k1zkp_v0_5_0_surjection_genrand(borromean_s, n_used_pubkeys, &blinding_key) == 0) {
+    rustsecp256k1zkp_v0_6_0_surjection_genmessage(msg32, ephemeral_input_tags, n_total_pubkeys, ephemeral_output_tag);
+    if (rustsecp256k1zkp_v0_6_0_surjection_genrand(borromean_s, n_used_pubkeys, &blinding_key) == 0) {
         return 0;
     }
     /* Borromean sign will overwrite one of the s values we just generated, so use
      * it as a nonce instead. This avoids extra random generation and also is an
      * homage to the rangeproof code which does this very cleverly to encode messages. */
     nonce = borromean_s[ring_input_index];
-    rustsecp256k1zkp_v0_5_0_scalar_clear(&borromean_s[ring_input_index]);
-    if (rustsecp256k1zkp_v0_5_0_borromean_sign(&ctx->ecmult_ctx, &ctx->ecmult_gen_ctx, &proof->data[0], borromean_s, ring_pubkeys, &nonce, &blinding_key, rsizes, indices, 1, msg32, 32) == 0) {
+    rustsecp256k1zkp_v0_6_0_scalar_clear(&borromean_s[ring_input_index]);
+    if (rustsecp256k1zkp_v0_6_0_borromean_sign(&ctx->ecmult_gen_ctx, &proof->data[0], borromean_s, ring_pubkeys, &nonce, &blinding_key, rsizes, indices, 1, msg32, 32) == 0) {
         return 0;
     }
     for (i = 0; i < n_used_pubkeys; i++) {
-        rustsecp256k1zkp_v0_5_0_scalar_get_b32(&proof->data[32 + 32 * i], &borromean_s[i]);
+        rustsecp256k1zkp_v0_6_0_scalar_get_b32(&proof->data[32 + 32 * i], &borromean_s[i]);
     }
     return 1;
 }
@@ -308,24 +307,23 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_generate(const rustsecp256k1zkp_v0_5
 #ifdef USE_REDUCED_SURJECTION_PROOF_SIZE
 static
 #endif
-int rustsecp256k1zkp_v0_5_0_surjectionproof_verify(const rustsecp256k1zkp_v0_5_0_context* ctx, const rustsecp256k1zkp_v0_5_0_surjectionproof* proof, const rustsecp256k1zkp_v0_5_0_generator* ephemeral_input_tags, size_t n_ephemeral_input_tags, const rustsecp256k1zkp_v0_5_0_generator* ephemeral_output_tag) {
+int rustsecp256k1zkp_v0_6_0_surjectionproof_verify(const rustsecp256k1zkp_v0_6_0_context* ctx, const rustsecp256k1zkp_v0_6_0_surjectionproof* proof, const rustsecp256k1zkp_v0_6_0_generator* ephemeral_input_tags, size_t n_ephemeral_input_tags, const rustsecp256k1zkp_v0_6_0_generator* ephemeral_output_tag) {
     size_t rsizes[1];    /* array needed for borromean sig API */
     size_t i;
     size_t n_total_pubkeys;
     size_t n_used_pubkeys;
-    rustsecp256k1zkp_v0_5_0_gej ring_pubkeys[SECP256K1_SURJECTIONPROOF_MAX_USED_INPUTS];
-    rustsecp256k1zkp_v0_5_0_scalar borromean_s[SECP256K1_SURJECTIONPROOF_MAX_USED_INPUTS];
+    rustsecp256k1zkp_v0_6_0_gej ring_pubkeys[SECP256K1_SURJECTIONPROOF_MAX_USED_INPUTS];
+    rustsecp256k1zkp_v0_6_0_scalar borromean_s[SECP256K1_SURJECTIONPROOF_MAX_USED_INPUTS];
     unsigned char msg32[32];
 
     VERIFY_CHECK(ctx != NULL);
-    ARG_CHECK(rustsecp256k1zkp_v0_5_0_ecmult_context_is_built(&ctx->ecmult_ctx));
     ARG_CHECK(proof != NULL);
     ARG_CHECK(ephemeral_input_tags != NULL);
     ARG_CHECK(ephemeral_output_tag != NULL);
 
     /* Compute public keys */
-    n_total_pubkeys = rustsecp256k1zkp_v0_5_0_surjectionproof_n_total_inputs(ctx, proof);
-    n_used_pubkeys = rustsecp256k1zkp_v0_5_0_surjectionproof_n_used_inputs(ctx, proof);
+    n_total_pubkeys = rustsecp256k1zkp_v0_6_0_surjectionproof_n_total_inputs(ctx, proof);
+    n_used_pubkeys = rustsecp256k1zkp_v0_6_0_surjectionproof_n_used_inputs(ctx, proof);
     if (n_used_pubkeys == 0 || n_used_pubkeys > n_total_pubkeys || n_total_pubkeys != n_ephemeral_input_tags) {
         return 0;
     }
@@ -335,7 +333,7 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_verify(const rustsecp256k1zkp_v0_5_0
         return 0;
     }
 
-    if (rustsecp256k1zkp_v0_5_0_surjection_compute_public_keys(ring_pubkeys, n_used_pubkeys, ephemeral_input_tags, n_total_pubkeys, proof->used_inputs, ephemeral_output_tag, 0, NULL) == 0) {
+    if (rustsecp256k1zkp_v0_6_0_surjection_compute_public_keys(ring_pubkeys, n_used_pubkeys, ephemeral_input_tags, n_total_pubkeys, proof->used_inputs, ephemeral_output_tag, 0, NULL) == 0) {
         return 0;
     }
 
@@ -343,13 +341,13 @@ int rustsecp256k1zkp_v0_5_0_surjectionproof_verify(const rustsecp256k1zkp_v0_5_0
     rsizes[0] = (int) n_used_pubkeys;
     for (i = 0; i < n_used_pubkeys; i++) {
         int overflow = 0;
-        rustsecp256k1zkp_v0_5_0_scalar_set_b32(&borromean_s[i], &proof->data[32 + 32 * i], &overflow);
+        rustsecp256k1zkp_v0_6_0_scalar_set_b32(&borromean_s[i], &proof->data[32 + 32 * i], &overflow);
         if (overflow == 1) {
             return 0;
         }
     }
-    rustsecp256k1zkp_v0_5_0_surjection_genmessage(msg32, ephemeral_input_tags, n_total_pubkeys, ephemeral_output_tag);
-    return rustsecp256k1zkp_v0_5_0_borromean_verify(&ctx->ecmult_ctx, NULL, &proof->data[0], borromean_s, ring_pubkeys, rsizes, 1, msg32, 32);
+    rustsecp256k1zkp_v0_6_0_surjection_genmessage(msg32, ephemeral_input_tags, n_total_pubkeys, ephemeral_output_tag);
+    return rustsecp256k1zkp_v0_6_0_borromean_verify(NULL, &proof->data[0], borromean_s, ring_pubkeys, rsizes, 1, msg32, 32);
 }
 
 #endif
