@@ -10,6 +10,8 @@
 #include "scalar.h"
 #include <string.h>
 
+#include "modinv64_impl.h"
+
 /* Limbs of the secp256k1 order. */
 #define SECP256K1_N_0 ((uint64_t)0xBFD25E8CD0364141ULL)
 #define SECP256K1_N_1 ((uint64_t)0xBAAEDCE6AF48A03BULL)
@@ -27,44 +29,44 @@
 #define SECP256K1_N_H_2 ((uint64_t)0xFFFFFFFFFFFFFFFFULL)
 #define SECP256K1_N_H_3 ((uint64_t)0x7FFFFFFFFFFFFFFFULL)
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_clear(rustsecp256k1zkp_v0_5_0_scalar *r) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_scalar_clear(rustsecp256k1zkp_v0_6_0_scalar *r) {
     r->d[0] = 0;
     r->d[1] = 0;
     r->d[2] = 0;
     r->d[3] = 0;
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_set_int(rustsecp256k1zkp_v0_5_0_scalar *r, unsigned int v) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_scalar_set_int(rustsecp256k1zkp_v0_6_0_scalar *r, unsigned int v) {
     r->d[0] = v;
     r->d[1] = 0;
     r->d[2] = 0;
     r->d[3] = 0;
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_set_u64(rustsecp256k1zkp_v0_5_0_scalar *r, uint64_t v) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_scalar_set_u64(rustsecp256k1zkp_v0_6_0_scalar *r, uint64_t v) {
     r->d[0] = v;
     r->d[1] = 0;
     r->d[2] = 0;
     r->d[3] = 0;
 }
 
-SECP256K1_INLINE static unsigned int rustsecp256k1zkp_v0_5_0_scalar_get_bits(const rustsecp256k1zkp_v0_5_0_scalar *a, unsigned int offset, unsigned int count) {
+SECP256K1_INLINE static unsigned int rustsecp256k1zkp_v0_6_0_scalar_get_bits(const rustsecp256k1zkp_v0_6_0_scalar *a, unsigned int offset, unsigned int count) {
     VERIFY_CHECK((offset + count - 1) >> 6 == offset >> 6);
     return (a->d[offset >> 6] >> (offset & 0x3F)) & ((((uint64_t)1) << count) - 1);
 }
 
-SECP256K1_INLINE static unsigned int rustsecp256k1zkp_v0_5_0_scalar_get_bits_var(const rustsecp256k1zkp_v0_5_0_scalar *a, unsigned int offset, unsigned int count) {
+SECP256K1_INLINE static unsigned int rustsecp256k1zkp_v0_6_0_scalar_get_bits_var(const rustsecp256k1zkp_v0_6_0_scalar *a, unsigned int offset, unsigned int count) {
     VERIFY_CHECK(count < 32);
     VERIFY_CHECK(offset + count <= 256);
     if ((offset + count - 1) >> 6 == offset >> 6) {
-        return rustsecp256k1zkp_v0_5_0_scalar_get_bits(a, offset, count);
+        return rustsecp256k1zkp_v0_6_0_scalar_get_bits(a, offset, count);
     } else {
         VERIFY_CHECK((offset >> 6) + 1 < 4);
         return ((a->d[offset >> 6] >> (offset & 0x3F)) | (a->d[(offset >> 6) + 1] << (64 - (offset & 0x3F)))) & ((((uint64_t)1) << count) - 1);
     }
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_check_overflow(const rustsecp256k1zkp_v0_5_0_scalar *a) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_check_overflow(const rustsecp256k1zkp_v0_6_0_scalar *a) {
     int yes = 0;
     int no = 0;
     no |= (a->d[3] < SECP256K1_N_3); /* No need for a > check. */
@@ -76,7 +78,7 @@ SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_check_overflow(const
     return yes;
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_reduce(rustsecp256k1zkp_v0_5_0_scalar *r, unsigned int overflow) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_reduce(rustsecp256k1zkp_v0_6_0_scalar *r, unsigned int overflow) {
     uint128_t t;
     VERIFY_CHECK(overflow <= 1);
     t = (uint128_t)r->d[0] + overflow * SECP256K1_N_C_0;
@@ -90,7 +92,7 @@ SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_reduce(rustsecp256k1z
     return overflow;
 }
 
-static int rustsecp256k1zkp_v0_5_0_scalar_add(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a, const rustsecp256k1zkp_v0_5_0_scalar *b) {
+static int rustsecp256k1zkp_v0_6_0_scalar_add(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *a, const rustsecp256k1zkp_v0_6_0_scalar *b) {
     int overflow;
     uint128_t t = (uint128_t)a->d[0] + b->d[0];
     r->d[0] = t & 0xFFFFFFFFFFFFFFFFULL; t >>= 64;
@@ -100,13 +102,13 @@ static int rustsecp256k1zkp_v0_5_0_scalar_add(rustsecp256k1zkp_v0_5_0_scalar *r,
     r->d[2] = t & 0xFFFFFFFFFFFFFFFFULL; t >>= 64;
     t += (uint128_t)a->d[3] + b->d[3];
     r->d[3] = t & 0xFFFFFFFFFFFFFFFFULL; t >>= 64;
-    overflow = t + rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r);
+    overflow = t + rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r);
     VERIFY_CHECK(overflow == 0 || overflow == 1);
-    rustsecp256k1zkp_v0_5_0_scalar_reduce(r, overflow);
+    rustsecp256k1zkp_v0_6_0_scalar_reduce(r, overflow);
     return overflow;
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_cadd_bit(rustsecp256k1zkp_v0_5_0_scalar *r, unsigned int bit, int flag) {
+static void rustsecp256k1zkp_v0_6_0_scalar_cadd_bit(rustsecp256k1zkp_v0_6_0_scalar *r, unsigned int bit, int flag) {
     uint128_t t;
     VERIFY_CHECK(bit < 256);
     bit += ((uint32_t) flag - 1) & 0x100;  /* forcing (bit >> 6) > 3 makes this a noop */
@@ -120,35 +122,35 @@ static void rustsecp256k1zkp_v0_5_0_scalar_cadd_bit(rustsecp256k1zkp_v0_5_0_scal
     r->d[3] = t & 0xFFFFFFFFFFFFFFFFULL;
 #ifdef VERIFY
     VERIFY_CHECK((t >> 64) == 0);
-    VERIFY_CHECK(rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r) == 0);
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r) == 0);
 #endif
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_set_b32(rustsecp256k1zkp_v0_5_0_scalar *r, const unsigned char *b32, int *overflow) {
+static void rustsecp256k1zkp_v0_6_0_scalar_set_b32(rustsecp256k1zkp_v0_6_0_scalar *r, const unsigned char *b32, int *overflow) {
     int over;
     r->d[0] = (uint64_t)b32[31] | (uint64_t)b32[30] << 8 | (uint64_t)b32[29] << 16 | (uint64_t)b32[28] << 24 | (uint64_t)b32[27] << 32 | (uint64_t)b32[26] << 40 | (uint64_t)b32[25] << 48 | (uint64_t)b32[24] << 56;
     r->d[1] = (uint64_t)b32[23] | (uint64_t)b32[22] << 8 | (uint64_t)b32[21] << 16 | (uint64_t)b32[20] << 24 | (uint64_t)b32[19] << 32 | (uint64_t)b32[18] << 40 | (uint64_t)b32[17] << 48 | (uint64_t)b32[16] << 56;
     r->d[2] = (uint64_t)b32[15] | (uint64_t)b32[14] << 8 | (uint64_t)b32[13] << 16 | (uint64_t)b32[12] << 24 | (uint64_t)b32[11] << 32 | (uint64_t)b32[10] << 40 | (uint64_t)b32[9] << 48 | (uint64_t)b32[8] << 56;
     r->d[3] = (uint64_t)b32[7] | (uint64_t)b32[6] << 8 | (uint64_t)b32[5] << 16 | (uint64_t)b32[4] << 24 | (uint64_t)b32[3] << 32 | (uint64_t)b32[2] << 40 | (uint64_t)b32[1] << 48 | (uint64_t)b32[0] << 56;
-    over = rustsecp256k1zkp_v0_5_0_scalar_reduce(r, rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r));
+    over = rustsecp256k1zkp_v0_6_0_scalar_reduce(r, rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r));
     if (overflow) {
         *overflow = over;
     }
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_get_b32(unsigned char *bin, const rustsecp256k1zkp_v0_5_0_scalar* a) {
+static void rustsecp256k1zkp_v0_6_0_scalar_get_b32(unsigned char *bin, const rustsecp256k1zkp_v0_6_0_scalar* a) {
     bin[0] = a->d[3] >> 56; bin[1] = a->d[3] >> 48; bin[2] = a->d[3] >> 40; bin[3] = a->d[3] >> 32; bin[4] = a->d[3] >> 24; bin[5] = a->d[3] >> 16; bin[6] = a->d[3] >> 8; bin[7] = a->d[3];
     bin[8] = a->d[2] >> 56; bin[9] = a->d[2] >> 48; bin[10] = a->d[2] >> 40; bin[11] = a->d[2] >> 32; bin[12] = a->d[2] >> 24; bin[13] = a->d[2] >> 16; bin[14] = a->d[2] >> 8; bin[15] = a->d[2];
     bin[16] = a->d[1] >> 56; bin[17] = a->d[1] >> 48; bin[18] = a->d[1] >> 40; bin[19] = a->d[1] >> 32; bin[20] = a->d[1] >> 24; bin[21] = a->d[1] >> 16; bin[22] = a->d[1] >> 8; bin[23] = a->d[1];
     bin[24] = a->d[0] >> 56; bin[25] = a->d[0] >> 48; bin[26] = a->d[0] >> 40; bin[27] = a->d[0] >> 32; bin[28] = a->d[0] >> 24; bin[29] = a->d[0] >> 16; bin[30] = a->d[0] >> 8; bin[31] = a->d[0];
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_is_zero(const rustsecp256k1zkp_v0_5_0_scalar *a) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_is_zero(const rustsecp256k1zkp_v0_6_0_scalar *a) {
     return (a->d[0] | a->d[1] | a->d[2] | a->d[3]) == 0;
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_negate(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a) {
-    uint64_t nonzero = 0xFFFFFFFFFFFFFFFFULL * (rustsecp256k1zkp_v0_5_0_scalar_is_zero(a) == 0);
+static void rustsecp256k1zkp_v0_6_0_scalar_negate(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *a) {
+    uint64_t nonzero = 0xFFFFFFFFFFFFFFFFULL * (rustsecp256k1zkp_v0_6_0_scalar_is_zero(a) == 0);
     uint128_t t = (uint128_t)(~a->d[0]) + SECP256K1_N_0 + 1;
     r->d[0] = t & nonzero; t >>= 64;
     t += (uint128_t)(~a->d[1]) + SECP256K1_N_1;
@@ -159,11 +161,11 @@ static void rustsecp256k1zkp_v0_5_0_scalar_negate(rustsecp256k1zkp_v0_5_0_scalar
     r->d[3] = t & nonzero;
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_is_one(const rustsecp256k1zkp_v0_5_0_scalar *a) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_is_one(const rustsecp256k1zkp_v0_6_0_scalar *a) {
     return ((a->d[0] ^ 1) | a->d[1] | a->d[2] | a->d[3]) == 0;
 }
 
-static int rustsecp256k1zkp_v0_5_0_scalar_is_high(const rustsecp256k1zkp_v0_5_0_scalar *a) {
+static int rustsecp256k1zkp_v0_6_0_scalar_is_high(const rustsecp256k1zkp_v0_6_0_scalar *a) {
     int yes = 0;
     int no = 0;
     no |= (a->d[3] < SECP256K1_N_H_3);
@@ -175,11 +177,11 @@ static int rustsecp256k1zkp_v0_5_0_scalar_is_high(const rustsecp256k1zkp_v0_5_0_
     return yes;
 }
 
-static int rustsecp256k1zkp_v0_5_0_scalar_cond_negate(rustsecp256k1zkp_v0_5_0_scalar *r, int flag) {
+static int rustsecp256k1zkp_v0_6_0_scalar_cond_negate(rustsecp256k1zkp_v0_6_0_scalar *r, int flag) {
     /* If we are flag = 0, mask = 00...00 and this is a no-op;
-     * if we are flag = 1, mask = 11...11 and this is identical to rustsecp256k1zkp_v0_5_0_scalar_negate */
+     * if we are flag = 1, mask = 11...11 and this is identical to rustsecp256k1zkp_v0_6_0_scalar_negate */
     uint64_t mask = !flag - 1;
-    uint64_t nonzero = (rustsecp256k1zkp_v0_5_0_scalar_is_zero(r) != 0) - 1;
+    uint64_t nonzero = (rustsecp256k1zkp_v0_6_0_scalar_is_zero(r) != 0) - 1;
     uint128_t t = (uint128_t)(r->d[0] ^ mask) + ((SECP256K1_N_0 + 1) & mask);
     r->d[0] = t & nonzero; t >>= 64;
     t += (uint128_t)(r->d[1] ^ mask) + (SECP256K1_N_1 & mask);
@@ -222,28 +224,6 @@ static int rustsecp256k1zkp_v0_5_0_scalar_cond_negate(rustsecp256k1zkp_v0_5_0_sc
     VERIFY_CHECK(c1 >= th); \
 }
 
-/** Add 2*a*b to the number defined by (c0,c1,c2). c2 must never overflow. */
-#define muladd2(a,b) { \
-    uint64_t tl, th, th2, tl2; \
-    { \
-        uint128_t t = (uint128_t)a * b; \
-        th = t >> 64;               /* at most 0xFFFFFFFFFFFFFFFE */ \
-        tl = t; \
-    } \
-    th2 = th + th;                  /* at most 0xFFFFFFFFFFFFFFFE (in case th was 0x7FFFFFFFFFFFFFFF) */ \
-    c2 += (th2 < th);               /* never overflows by contract (verified the next line) */ \
-    VERIFY_CHECK((th2 >= th) || (c2 != 0)); \
-    tl2 = tl + tl;                  /* at most 0xFFFFFFFFFFFFFFFE (in case the lowest 63 bits of tl were 0x7FFFFFFFFFFFFFFF) */ \
-    th2 += (tl2 < tl);              /* at most 0xFFFFFFFFFFFFFFFF */ \
-    c0 += tl2;                      /* overflow is handled on the next line */ \
-    th2 += (c0 < tl2);              /* second overflow is handled on the next line */ \
-    c2 += (c0 < tl2) & (th2 == 0);  /* never overflows by contract (verified the next line) */ \
-    VERIFY_CHECK((c0 >= tl2) || (th2 != 0) || (c2 != 0)); \
-    c1 += th2;                      /* overflow is handled on the next line */ \
-    c2 += (c1 < th2);               /* never overflows by contract (verified the next line) */ \
-    VERIFY_CHECK((c1 >= th2) || (c2 != 0)); \
-}
-
 /** Add a to the number defined by (c0,c1,c2). c2 must never overflow. */
 #define sumadd(a) { \
     unsigned int over; \
@@ -277,7 +257,7 @@ static int rustsecp256k1zkp_v0_5_0_scalar_cond_negate(rustsecp256k1zkp_v0_5_0_sc
     VERIFY_CHECK(c2 == 0); \
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_reduce_512(rustsecp256k1zkp_v0_5_0_scalar *r, const uint64_t *l) {
+static void rustsecp256k1zkp_v0_6_0_scalar_reduce_512(rustsecp256k1zkp_v0_6_0_scalar *r, const uint64_t *l) {
 #ifdef USE_ASM_X86_64
     /* Reduce 512 bits into 385. */
     uint64_t m0, m1, m2, m3, m4, m5, m6;
@@ -583,10 +563,10 @@ static void rustsecp256k1zkp_v0_5_0_scalar_reduce_512(rustsecp256k1zkp_v0_5_0_sc
 #endif
 
     /* Final reduction of r. */
-    rustsecp256k1zkp_v0_5_0_scalar_reduce(r, c + rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r));
+    rustsecp256k1zkp_v0_6_0_scalar_reduce(r, c + rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r));
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_mul_512(uint64_t l[8], const rustsecp256k1zkp_v0_5_0_scalar *a, const rustsecp256k1zkp_v0_5_0_scalar *b) {
+static void rustsecp256k1zkp_v0_6_0_scalar_mul_512(uint64_t l[8], const rustsecp256k1zkp_v0_6_0_scalar *a, const rustsecp256k1zkp_v0_6_0_scalar *b) {
 #ifdef USE_ASM_X86_64
     const uint64_t *pb = b->d;
     __asm__ __volatile__(
@@ -753,158 +733,20 @@ static void rustsecp256k1zkp_v0_5_0_scalar_mul_512(uint64_t l[8], const rustsecp
 #endif
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_sqr_512(uint64_t l[8], const rustsecp256k1zkp_v0_5_0_scalar *a) {
-#ifdef USE_ASM_X86_64
-    __asm__ __volatile__(
-    /* Preload */
-    "movq 0(%%rdi), %%r11\n"
-    "movq 8(%%rdi), %%r12\n"
-    "movq 16(%%rdi), %%r13\n"
-    "movq 24(%%rdi), %%r14\n"
-    /* (rax,rdx) = a0 * a0 */
-    "movq %%r11, %%rax\n"
-    "mulq %%r11\n"
-    /* Extract l0 */
-    "movq %%rax, 0(%%rsi)\n"
-    /* (r8,r9,r10) = (rdx,0) */
-    "movq %%rdx, %%r8\n"
-    "xorq %%r9, %%r9\n"
-    "xorq %%r10, %%r10\n"
-    /* (r8,r9,r10) += 2 * a0 * a1 */
-    "movq %%r11, %%rax\n"
-    "mulq %%r12\n"
-    "addq %%rax, %%r8\n"
-    "adcq %%rdx, %%r9\n"
-    "adcq $0, %%r10\n"
-    "addq %%rax, %%r8\n"
-    "adcq %%rdx, %%r9\n"
-    "adcq $0, %%r10\n"
-    /* Extract l1 */
-    "movq %%r8, 8(%%rsi)\n"
-    "xorq %%r8, %%r8\n"
-    /* (r9,r10,r8) += 2 * a0 * a2 */
-    "movq %%r11, %%rax\n"
-    "mulq %%r13\n"
-    "addq %%rax, %%r9\n"
-    "adcq %%rdx, %%r10\n"
-    "adcq $0, %%r8\n"
-    "addq %%rax, %%r9\n"
-    "adcq %%rdx, %%r10\n"
-    "adcq $0, %%r8\n"
-    /* (r9,r10,r8) += a1 * a1 */
-    "movq %%r12, %%rax\n"
-    "mulq %%r12\n"
-    "addq %%rax, %%r9\n"
-    "adcq %%rdx, %%r10\n"
-    "adcq $0, %%r8\n"
-    /* Extract l2 */
-    "movq %%r9, 16(%%rsi)\n"
-    "xorq %%r9, %%r9\n"
-    /* (r10,r8,r9) += 2 * a0 * a3 */
-    "movq %%r11, %%rax\n"
-    "mulq %%r14\n"
-    "addq %%rax, %%r10\n"
-    "adcq %%rdx, %%r8\n"
-    "adcq $0, %%r9\n"
-    "addq %%rax, %%r10\n"
-    "adcq %%rdx, %%r8\n"
-    "adcq $0, %%r9\n"
-    /* (r10,r8,r9) += 2 * a1 * a2 */
-    "movq %%r12, %%rax\n"
-    "mulq %%r13\n"
-    "addq %%rax, %%r10\n"
-    "adcq %%rdx, %%r8\n"
-    "adcq $0, %%r9\n"
-    "addq %%rax, %%r10\n"
-    "adcq %%rdx, %%r8\n"
-    "adcq $0, %%r9\n"
-    /* Extract l3 */
-    "movq %%r10, 24(%%rsi)\n"
-    "xorq %%r10, %%r10\n"
-    /* (r8,r9,r10) += 2 * a1 * a3 */
-    "movq %%r12, %%rax\n"
-    "mulq %%r14\n"
-    "addq %%rax, %%r8\n"
-    "adcq %%rdx, %%r9\n"
-    "adcq $0, %%r10\n"
-    "addq %%rax, %%r8\n"
-    "adcq %%rdx, %%r9\n"
-    "adcq $0, %%r10\n"
-    /* (r8,r9,r10) += a2 * a2 */
-    "movq %%r13, %%rax\n"
-    "mulq %%r13\n"
-    "addq %%rax, %%r8\n"
-    "adcq %%rdx, %%r9\n"
-    "adcq $0, %%r10\n"
-    /* Extract l4 */
-    "movq %%r8, 32(%%rsi)\n"
-    "xorq %%r8, %%r8\n"
-    /* (r9,r10,r8) += 2 * a2 * a3 */
-    "movq %%r13, %%rax\n"
-    "mulq %%r14\n"
-    "addq %%rax, %%r9\n"
-    "adcq %%rdx, %%r10\n"
-    "adcq $0, %%r8\n"
-    "addq %%rax, %%r9\n"
-    "adcq %%rdx, %%r10\n"
-    "adcq $0, %%r8\n"
-    /* Extract l5 */
-    "movq %%r9, 40(%%rsi)\n"
-    /* (r10,r8) += a3 * a3 */
-    "movq %%r14, %%rax\n"
-    "mulq %%r14\n"
-    "addq %%rax, %%r10\n"
-    "adcq %%rdx, %%r8\n"
-    /* Extract l6 */
-    "movq %%r10, 48(%%rsi)\n"
-    /* Extract l7 */
-    "movq %%r8, 56(%%rsi)\n"
-    :
-    : "S"(l), "D"(a->d)
-    : "rax", "rdx", "r8", "r9", "r10", "r11", "r12", "r13", "r14", "cc", "memory");
-#else
-    /* 160 bit accumulator. */
-    uint64_t c0 = 0, c1 = 0;
-    uint32_t c2 = 0;
-
-    /* l[0..7] = a[0..3] * b[0..3]. */
-    muladd_fast(a->d[0], a->d[0]);
-    extract_fast(l[0]);
-    muladd2(a->d[0], a->d[1]);
-    extract(l[1]);
-    muladd2(a->d[0], a->d[2]);
-    muladd(a->d[1], a->d[1]);
-    extract(l[2]);
-    muladd2(a->d[0], a->d[3]);
-    muladd2(a->d[1], a->d[2]);
-    extract(l[3]);
-    muladd2(a->d[1], a->d[3]);
-    muladd(a->d[2], a->d[2]);
-    extract(l[4]);
-    muladd2(a->d[2], a->d[3]);
-    extract(l[5]);
-    muladd_fast(a->d[3], a->d[3]);
-    extract_fast(l[6]);
-    VERIFY_CHECK(c1 == 0);
-    l[7] = c0;
-#endif
-}
-
 #undef sumadd
 #undef sumadd_fast
 #undef muladd
 #undef muladd_fast
-#undef muladd2
 #undef extract
 #undef extract_fast
 
-static void rustsecp256k1zkp_v0_5_0_scalar_mul(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a, const rustsecp256k1zkp_v0_5_0_scalar *b) {
+static void rustsecp256k1zkp_v0_6_0_scalar_mul(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *a, const rustsecp256k1zkp_v0_6_0_scalar *b) {
     uint64_t l[8];
-    rustsecp256k1zkp_v0_5_0_scalar_mul_512(l, a, b);
-    rustsecp256k1zkp_v0_5_0_scalar_reduce_512(r, l);
+    rustsecp256k1zkp_v0_6_0_scalar_mul_512(l, a, b);
+    rustsecp256k1zkp_v0_6_0_scalar_reduce_512(r, l);
 }
 
-static int rustsecp256k1zkp_v0_5_0_scalar_shr_int(rustsecp256k1zkp_v0_5_0_scalar *r, int n) {
+static int rustsecp256k1zkp_v0_6_0_scalar_shr_int(rustsecp256k1zkp_v0_6_0_scalar *r, int n) {
     int ret;
     VERIFY_CHECK(n > 0);
     VERIFY_CHECK(n < 16);
@@ -916,13 +758,7 @@ static int rustsecp256k1zkp_v0_5_0_scalar_shr_int(rustsecp256k1zkp_v0_5_0_scalar
     return ret;
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_sqr(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a) {
-    uint64_t l[8];
-    rustsecp256k1zkp_v0_5_0_scalar_sqr_512(l, a);
-    rustsecp256k1zkp_v0_5_0_scalar_reduce_512(r, l);
-}
-
-static void rustsecp256k1zkp_v0_5_0_scalar_split_128(rustsecp256k1zkp_v0_5_0_scalar *r1, rustsecp256k1zkp_v0_5_0_scalar *r2, const rustsecp256k1zkp_v0_5_0_scalar *k) {
+static void rustsecp256k1zkp_v0_6_0_scalar_split_128(rustsecp256k1zkp_v0_6_0_scalar *r1, rustsecp256k1zkp_v0_6_0_scalar *r2, const rustsecp256k1zkp_v0_6_0_scalar *k) {
     r1->d[0] = k->d[0];
     r1->d[1] = k->d[1];
     r1->d[2] = 0;
@@ -933,17 +769,17 @@ static void rustsecp256k1zkp_v0_5_0_scalar_split_128(rustsecp256k1zkp_v0_5_0_sca
     r2->d[3] = 0;
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_eq(const rustsecp256k1zkp_v0_5_0_scalar *a, const rustsecp256k1zkp_v0_5_0_scalar *b) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_eq(const rustsecp256k1zkp_v0_6_0_scalar *a, const rustsecp256k1zkp_v0_6_0_scalar *b) {
     return ((a->d[0] ^ b->d[0]) | (a->d[1] ^ b->d[1]) | (a->d[2] ^ b->d[2]) | (a->d[3] ^ b->d[3])) == 0;
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_mul_shift_var(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a, const rustsecp256k1zkp_v0_5_0_scalar *b, unsigned int shift) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_scalar_mul_shift_var(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *a, const rustsecp256k1zkp_v0_6_0_scalar *b, unsigned int shift) {
     uint64_t l[8];
     unsigned int shiftlimbs;
     unsigned int shiftlow;
     unsigned int shifthigh;
     VERIFY_CHECK(shift >= 256);
-    rustsecp256k1zkp_v0_5_0_scalar_mul_512(l, a, b);
+    rustsecp256k1zkp_v0_6_0_scalar_mul_512(l, a, b);
     shiftlimbs = shift >> 6;
     shiftlow = shift & 0x3F;
     shifthigh = 64 - shiftlow;
@@ -951,10 +787,10 @@ SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_mul_shift_var(rustse
     r->d[1] = shift < 448 ? (l[1 + shiftlimbs] >> shiftlow | (shift < 384 && shiftlow ? (l[2 + shiftlimbs] << shifthigh) : 0)) : 0;
     r->d[2] = shift < 384 ? (l[2 + shiftlimbs] >> shiftlow | (shift < 320 && shiftlow ? (l[3 + shiftlimbs] << shifthigh) : 0)) : 0;
     r->d[3] = shift < 320 ? (l[3 + shiftlimbs] >> shiftlow) : 0;
-    rustsecp256k1zkp_v0_5_0_scalar_cadd_bit(r, 0, (l[(shift - 1) >> 6] >> ((shift - 1) & 0x3f)) & 1);
+    rustsecp256k1zkp_v0_6_0_scalar_cadd_bit(r, 0, (l[(shift - 1) >> 6] >> ((shift - 1) & 0x3f)) & 1);
 }
 
-static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_scalar_cmov(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a, int flag) {
+static SECP256K1_INLINE void rustsecp256k1zkp_v0_6_0_scalar_cmov(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *a, int flag) {
     uint64_t mask0, mask1;
     VG_CHECK_VERIFY(r->d, sizeof(r->d));
     mask0 = flag + ~((uint64_t)0);
@@ -978,7 +814,7 @@ static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_scalar_cmov(rustsecp256k1zk
 #define LE32(p) (p)
 #endif
 
-static void rustsecp256k1zkp_v0_5_0_scalar_chacha20(rustsecp256k1zkp_v0_5_0_scalar *r1, rustsecp256k1zkp_v0_5_0_scalar *r2, const unsigned char *seed, uint64_t idx) {
+static void rustsecp256k1zkp_v0_6_0_scalar_chacha20(rustsecp256k1zkp_v0_6_0_scalar *r1, rustsecp256k1zkp_v0_6_0_scalar *r2, const unsigned char *seed, uint64_t idx) {
     size_t n;
     size_t over_count = 0;
     uint32_t seed32[8];
@@ -1042,8 +878,8 @@ static void rustsecp256k1zkp_v0_5_0_scalar_chacha20(rustsecp256k1zkp_v0_5_0_scal
         r2->d[1] = (((uint64_t) x12) << 32) | x13;
         r2->d[0] = (((uint64_t) x14) << 32) | x15;
 
-        over1 = rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r1);
-        over2 = rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r2);
+        over1 = rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r1);
+        over2 = rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r2);
         over_count++;
    } while (over1 | over2);
 }
@@ -1052,4 +888,78 @@ static void rustsecp256k1zkp_v0_5_0_scalar_chacha20(rustsecp256k1zkp_v0_5_0_scal
 #undef QUARTERROUND
 #undef LE32
 
+static void rustsecp256k1zkp_v0_6_0_scalar_from_signed62(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_modinv64_signed62 *a) {
+    const uint64_t a0 = a->v[0], a1 = a->v[1], a2 = a->v[2], a3 = a->v[3], a4 = a->v[4];
+
+    /* The output from rustsecp256k1zkp_v0_6_0_modinv64{_var} should be normalized to range [0,modulus), and
+     * have limbs in [0,2^62). The modulus is < 2^256, so the top limb must be below 2^(256-62*4).
+     */
+    VERIFY_CHECK(a0 >> 62 == 0);
+    VERIFY_CHECK(a1 >> 62 == 0);
+    VERIFY_CHECK(a2 >> 62 == 0);
+    VERIFY_CHECK(a3 >> 62 == 0);
+    VERIFY_CHECK(a4 >> 8 == 0);
+
+    r->d[0] = a0      | a1 << 62;
+    r->d[1] = a1 >> 2 | a2 << 60;
+    r->d[2] = a2 >> 4 | a3 << 58;
+    r->d[3] = a3 >> 6 | a4 << 56;
+
+#ifdef VERIFY
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r) == 0);
+#endif
+}
+
+static void rustsecp256k1zkp_v0_6_0_scalar_to_signed62(rustsecp256k1zkp_v0_6_0_modinv64_signed62 *r, const rustsecp256k1zkp_v0_6_0_scalar *a) {
+    const uint64_t M62 = UINT64_MAX >> 2;
+    const uint64_t a0 = a->d[0], a1 = a->d[1], a2 = a->d[2], a3 = a->d[3];
+
+#ifdef VERIFY
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_scalar_check_overflow(a) == 0);
+#endif
+
+    r->v[0] =  a0                   & M62;
+    r->v[1] = (a0 >> 62 | a1 <<  2) & M62;
+    r->v[2] = (a1 >> 60 | a2 <<  4) & M62;
+    r->v[3] = (a2 >> 58 | a3 <<  6) & M62;
+    r->v[4] =  a3 >> 56;
+}
+
+static const rustsecp256k1zkp_v0_6_0_modinv64_modinfo rustsecp256k1zkp_v0_6_0_const_modinfo_scalar = {
+    {{0x3FD25E8CD0364141LL, 0x2ABB739ABD2280EELL, -0x15LL, 0, 256}},
+    0x34F20099AA774EC1LL
+};
+
+static void rustsecp256k1zkp_v0_6_0_scalar_inverse(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *x) {
+    rustsecp256k1zkp_v0_6_0_modinv64_signed62 s;
+#ifdef VERIFY
+    int zero_in = rustsecp256k1zkp_v0_6_0_scalar_is_zero(x);
+#endif
+    rustsecp256k1zkp_v0_6_0_scalar_to_signed62(&s, x);
+    rustsecp256k1zkp_v0_6_0_modinv64(&s, &rustsecp256k1zkp_v0_6_0_const_modinfo_scalar);
+    rustsecp256k1zkp_v0_6_0_scalar_from_signed62(r, &s);
+
+#ifdef VERIFY
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_scalar_is_zero(r) == zero_in);
+#endif
+}
+
+static void rustsecp256k1zkp_v0_6_0_scalar_inverse_var(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *x) {
+    rustsecp256k1zkp_v0_6_0_modinv64_signed62 s;
+#ifdef VERIFY
+    int zero_in = rustsecp256k1zkp_v0_6_0_scalar_is_zero(x);
+#endif
+    rustsecp256k1zkp_v0_6_0_scalar_to_signed62(&s, x);
+    rustsecp256k1zkp_v0_6_0_modinv64_var(&s, &rustsecp256k1zkp_v0_6_0_const_modinfo_scalar);
+    rustsecp256k1zkp_v0_6_0_scalar_from_signed62(r, &s);
+
+#ifdef VERIFY
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_scalar_is_zero(r) == zero_in);
+#endif
+}
+
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_is_even(const rustsecp256k1zkp_v0_6_0_scalar *a) {
+    return !(a->d[0] & 1);
+}
+
 #endif /* SECP256K1_SCALAR_REPR_IMPL_H */
