@@ -9,6 +9,8 @@
 
 #include <string.h>
 
+#include "modinv32_impl.h"
+
 /* Limbs of the secp256k1 order. */
 #define SECP256K1_N_0 ((uint32_t)0xD0364141UL)
 #define SECP256K1_N_1 ((uint32_t)0xBFD25E8CUL)
@@ -36,7 +38,7 @@
 #define SECP256K1_N_H_6 ((uint32_t)0xFFFFFFFFUL)
 #define SECP256K1_N_H_7 ((uint32_t)0x7FFFFFFFUL)
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_clear(rustsecp256k1zkp_v0_5_0_scalar *r) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_scalar_clear(rustsecp256k1zkp_v0_6_0_scalar *r) {
     r->d[0] = 0;
     r->d[1] = 0;
     r->d[2] = 0;
@@ -47,7 +49,7 @@ SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_clear(rustsecp256k1z
     r->d[7] = 0;
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_set_int(rustsecp256k1zkp_v0_5_0_scalar *r, unsigned int v) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_scalar_set_int(rustsecp256k1zkp_v0_6_0_scalar *r, unsigned int v) {
     r->d[0] = v;
     r->d[1] = 0;
     r->d[2] = 0;
@@ -58,7 +60,7 @@ SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_set_int(rustsecp256k
     r->d[7] = 0;
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_set_u64(rustsecp256k1zkp_v0_5_0_scalar *r, uint64_t v) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_scalar_set_u64(rustsecp256k1zkp_v0_6_0_scalar *r, uint64_t v) {
     r->d[0] = v;
     r->d[1] = v >> 32;
     r->d[2] = 0;
@@ -69,23 +71,23 @@ SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_set_u64(rustsecp256k
     r->d[7] = 0;
 }
 
-SECP256K1_INLINE static unsigned int rustsecp256k1zkp_v0_5_0_scalar_get_bits(const rustsecp256k1zkp_v0_5_0_scalar *a, unsigned int offset, unsigned int count) {
+SECP256K1_INLINE static unsigned int rustsecp256k1zkp_v0_6_0_scalar_get_bits(const rustsecp256k1zkp_v0_6_0_scalar *a, unsigned int offset, unsigned int count) {
     VERIFY_CHECK((offset + count - 1) >> 5 == offset >> 5);
     return (a->d[offset >> 5] >> (offset & 0x1F)) & ((1 << count) - 1);
 }
 
-SECP256K1_INLINE static unsigned int rustsecp256k1zkp_v0_5_0_scalar_get_bits_var(const rustsecp256k1zkp_v0_5_0_scalar *a, unsigned int offset, unsigned int count) {
+SECP256K1_INLINE static unsigned int rustsecp256k1zkp_v0_6_0_scalar_get_bits_var(const rustsecp256k1zkp_v0_6_0_scalar *a, unsigned int offset, unsigned int count) {
     VERIFY_CHECK(count < 32);
     VERIFY_CHECK(offset + count <= 256);
     if ((offset + count - 1) >> 5 == offset >> 5) {
-        return rustsecp256k1zkp_v0_5_0_scalar_get_bits(a, offset, count);
+        return rustsecp256k1zkp_v0_6_0_scalar_get_bits(a, offset, count);
     } else {
         VERIFY_CHECK((offset >> 5) + 1 < 8);
         return ((a->d[offset >> 5] >> (offset & 0x1F)) | (a->d[(offset >> 5) + 1] << (32 - (offset & 0x1F)))) & ((((uint32_t)1) << count) - 1);
     }
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_check_overflow(const rustsecp256k1zkp_v0_5_0_scalar *a) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_check_overflow(const rustsecp256k1zkp_v0_6_0_scalar *a) {
     int yes = 0;
     int no = 0;
     no |= (a->d[7] < SECP256K1_N_7); /* No need for a > check. */
@@ -103,7 +105,7 @@ SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_check_overflow(const
     return yes;
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_reduce(rustsecp256k1zkp_v0_5_0_scalar *r, uint32_t overflow) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_reduce(rustsecp256k1zkp_v0_6_0_scalar *r, uint32_t overflow) {
     uint64_t t;
     VERIFY_CHECK(overflow <= 1);
     t = (uint64_t)r->d[0] + overflow * SECP256K1_N_C_0;
@@ -125,7 +127,7 @@ SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_reduce(rustsecp256k1z
     return overflow;
 }
 
-static int rustsecp256k1zkp_v0_5_0_scalar_add(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a, const rustsecp256k1zkp_v0_5_0_scalar *b) {
+static int rustsecp256k1zkp_v0_6_0_scalar_add(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *a, const rustsecp256k1zkp_v0_6_0_scalar *b) {
     int overflow;
     uint64_t t = (uint64_t)a->d[0] + b->d[0];
     r->d[0] = t & 0xFFFFFFFFULL; t >>= 32;
@@ -143,13 +145,13 @@ static int rustsecp256k1zkp_v0_5_0_scalar_add(rustsecp256k1zkp_v0_5_0_scalar *r,
     r->d[6] = t & 0xFFFFFFFFULL; t >>= 32;
     t += (uint64_t)a->d[7] + b->d[7];
     r->d[7] = t & 0xFFFFFFFFULL; t >>= 32;
-    overflow = t + rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r);
+    overflow = t + rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r);
     VERIFY_CHECK(overflow == 0 || overflow == 1);
-    rustsecp256k1zkp_v0_5_0_scalar_reduce(r, overflow);
+    rustsecp256k1zkp_v0_6_0_scalar_reduce(r, overflow);
     return overflow;
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_cadd_bit(rustsecp256k1zkp_v0_5_0_scalar *r, unsigned int bit, int flag) {
+static void rustsecp256k1zkp_v0_6_0_scalar_cadd_bit(rustsecp256k1zkp_v0_6_0_scalar *r, unsigned int bit, int flag) {
     uint64_t t;
     VERIFY_CHECK(bit < 256);
     bit += ((uint32_t) flag - 1) & 0x100;  /* forcing (bit >> 5) > 7 makes this a noop */
@@ -171,11 +173,11 @@ static void rustsecp256k1zkp_v0_5_0_scalar_cadd_bit(rustsecp256k1zkp_v0_5_0_scal
     r->d[7] = t & 0xFFFFFFFFULL;
 #ifdef VERIFY
     VERIFY_CHECK((t >> 32) == 0);
-    VERIFY_CHECK(rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r) == 0);
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r) == 0);
 #endif
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_set_b32(rustsecp256k1zkp_v0_5_0_scalar *r, const unsigned char *b32, int *overflow) {
+static void rustsecp256k1zkp_v0_6_0_scalar_set_b32(rustsecp256k1zkp_v0_6_0_scalar *r, const unsigned char *b32, int *overflow) {
     int over;
     r->d[0] = (uint32_t)b32[31] | (uint32_t)b32[30] << 8 | (uint32_t)b32[29] << 16 | (uint32_t)b32[28] << 24;
     r->d[1] = (uint32_t)b32[27] | (uint32_t)b32[26] << 8 | (uint32_t)b32[25] << 16 | (uint32_t)b32[24] << 24;
@@ -185,13 +187,13 @@ static void rustsecp256k1zkp_v0_5_0_scalar_set_b32(rustsecp256k1zkp_v0_5_0_scala
     r->d[5] = (uint32_t)b32[11] | (uint32_t)b32[10] << 8 | (uint32_t)b32[9] << 16 | (uint32_t)b32[8] << 24;
     r->d[6] = (uint32_t)b32[7] | (uint32_t)b32[6] << 8 | (uint32_t)b32[5] << 16 | (uint32_t)b32[4] << 24;
     r->d[7] = (uint32_t)b32[3] | (uint32_t)b32[2] << 8 | (uint32_t)b32[1] << 16 | (uint32_t)b32[0] << 24;
-    over = rustsecp256k1zkp_v0_5_0_scalar_reduce(r, rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r));
+    over = rustsecp256k1zkp_v0_6_0_scalar_reduce(r, rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r));
     if (overflow) {
         *overflow = over;
     }
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_get_b32(unsigned char *bin, const rustsecp256k1zkp_v0_5_0_scalar* a) {
+static void rustsecp256k1zkp_v0_6_0_scalar_get_b32(unsigned char *bin, const rustsecp256k1zkp_v0_6_0_scalar* a) {
     bin[0] = a->d[7] >> 24; bin[1] = a->d[7] >> 16; bin[2] = a->d[7] >> 8; bin[3] = a->d[7];
     bin[4] = a->d[6] >> 24; bin[5] = a->d[6] >> 16; bin[6] = a->d[6] >> 8; bin[7] = a->d[6];
     bin[8] = a->d[5] >> 24; bin[9] = a->d[5] >> 16; bin[10] = a->d[5] >> 8; bin[11] = a->d[5];
@@ -202,12 +204,12 @@ static void rustsecp256k1zkp_v0_5_0_scalar_get_b32(unsigned char *bin, const rus
     bin[28] = a->d[0] >> 24; bin[29] = a->d[0] >> 16; bin[30] = a->d[0] >> 8; bin[31] = a->d[0];
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_is_zero(const rustsecp256k1zkp_v0_5_0_scalar *a) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_is_zero(const rustsecp256k1zkp_v0_6_0_scalar *a) {
     return (a->d[0] | a->d[1] | a->d[2] | a->d[3] | a->d[4] | a->d[5] | a->d[6] | a->d[7]) == 0;
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_negate(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a) {
-    uint32_t nonzero = 0xFFFFFFFFUL * (rustsecp256k1zkp_v0_5_0_scalar_is_zero(a) == 0);
+static void rustsecp256k1zkp_v0_6_0_scalar_negate(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *a) {
+    uint32_t nonzero = 0xFFFFFFFFUL * (rustsecp256k1zkp_v0_6_0_scalar_is_zero(a) == 0);
     uint64_t t = (uint64_t)(~a->d[0]) + SECP256K1_N_0 + 1;
     r->d[0] = t & nonzero; t >>= 32;
     t += (uint64_t)(~a->d[1]) + SECP256K1_N_1;
@@ -226,11 +228,11 @@ static void rustsecp256k1zkp_v0_5_0_scalar_negate(rustsecp256k1zkp_v0_5_0_scalar
     r->d[7] = t & nonzero;
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_is_one(const rustsecp256k1zkp_v0_5_0_scalar *a) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_is_one(const rustsecp256k1zkp_v0_6_0_scalar *a) {
     return ((a->d[0] ^ 1) | a->d[1] | a->d[2] | a->d[3] | a->d[4] | a->d[5] | a->d[6] | a->d[7]) == 0;
 }
 
-static int rustsecp256k1zkp_v0_5_0_scalar_is_high(const rustsecp256k1zkp_v0_5_0_scalar *a) {
+static int rustsecp256k1zkp_v0_6_0_scalar_is_high(const rustsecp256k1zkp_v0_6_0_scalar *a) {
     int yes = 0;
     int no = 0;
     no |= (a->d[7] < SECP256K1_N_H_7);
@@ -248,11 +250,11 @@ static int rustsecp256k1zkp_v0_5_0_scalar_is_high(const rustsecp256k1zkp_v0_5_0_
     return yes;
 }
 
-static int rustsecp256k1zkp_v0_5_0_scalar_cond_negate(rustsecp256k1zkp_v0_5_0_scalar *r, int flag) {
+static int rustsecp256k1zkp_v0_6_0_scalar_cond_negate(rustsecp256k1zkp_v0_6_0_scalar *r, int flag) {
     /* If we are flag = 0, mask = 00...00 and this is a no-op;
-     * if we are flag = 1, mask = 11...11 and this is identical to rustsecp256k1zkp_v0_5_0_scalar_negate */
+     * if we are flag = 1, mask = 11...11 and this is identical to rustsecp256k1zkp_v0_6_0_scalar_negate */
     uint32_t mask = !flag - 1;
-    uint32_t nonzero = 0xFFFFFFFFUL * (rustsecp256k1zkp_v0_5_0_scalar_is_zero(r) == 0);
+    uint32_t nonzero = 0xFFFFFFFFUL * (rustsecp256k1zkp_v0_6_0_scalar_is_zero(r) == 0);
     uint64_t t = (uint64_t)(r->d[0] ^ mask) + ((SECP256K1_N_0 + 1) & mask);
     r->d[0] = t & nonzero; t >>= 32;
     t += (uint64_t)(r->d[1] ^ mask) + (SECP256K1_N_1 & mask);
@@ -304,28 +306,6 @@ static int rustsecp256k1zkp_v0_5_0_scalar_cond_negate(rustsecp256k1zkp_v0_5_0_sc
     VERIFY_CHECK(c1 >= th); \
 }
 
-/** Add 2*a*b to the number defined by (c0,c1,c2). c2 must never overflow. */
-#define muladd2(a,b) { \
-    uint32_t tl, th, th2, tl2; \
-    { \
-        uint64_t t = (uint64_t)a * b; \
-        th = t >> 32;               /* at most 0xFFFFFFFE */ \
-        tl = t; \
-    } \
-    th2 = th + th;                  /* at most 0xFFFFFFFE (in case th was 0x7FFFFFFF) */ \
-    c2 += (th2 < th);               /* never overflows by contract (verified the next line) */ \
-    VERIFY_CHECK((th2 >= th) || (c2 != 0)); \
-    tl2 = tl + tl;                  /* at most 0xFFFFFFFE (in case the lowest 63 bits of tl were 0x7FFFFFFF) */ \
-    th2 += (tl2 < tl);              /* at most 0xFFFFFFFF */ \
-    c0 += tl2;                      /* overflow is handled on the next line */ \
-    th2 += (c0 < tl2);              /* second overflow is handled on the next line */ \
-    c2 += (c0 < tl2) & (th2 == 0);  /* never overflows by contract (verified the next line) */ \
-    VERIFY_CHECK((c0 >= tl2) || (th2 != 0) || (c2 != 0)); \
-    c1 += th2;                      /* overflow is handled on the next line */ \
-    c2 += (c1 < th2);               /* never overflows by contract (verified the next line) */ \
-    VERIFY_CHECK((c1 >= th2) || (c2 != 0)); \
-}
-
 /** Add a to the number defined by (c0,c1,c2). c2 must never overflow. */
 #define sumadd(a) { \
     unsigned int over; \
@@ -359,7 +339,7 @@ static int rustsecp256k1zkp_v0_5_0_scalar_cond_negate(rustsecp256k1zkp_v0_5_0_sc
     VERIFY_CHECK(c2 == 0); \
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_reduce_512(rustsecp256k1zkp_v0_5_0_scalar *r, const uint32_t *l) {
+static void rustsecp256k1zkp_v0_6_0_scalar_reduce_512(rustsecp256k1zkp_v0_6_0_scalar *r, const uint32_t *l) {
     uint64_t c;
     uint32_t n0 = l[8], n1 = l[9], n2 = l[10], n3 = l[11], n4 = l[12], n5 = l[13], n6 = l[14], n7 = l[15];
     uint32_t m0, m1, m2, m3, m4, m5, m6, m7, m8, m9, m10, m11, m12;
@@ -498,10 +478,10 @@ static void rustsecp256k1zkp_v0_5_0_scalar_reduce_512(rustsecp256k1zkp_v0_5_0_sc
     r->d[7] = c & 0xFFFFFFFFUL; c >>= 32;
 
     /* Final reduction of r. */
-    rustsecp256k1zkp_v0_5_0_scalar_reduce(r, c + rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r));
+    rustsecp256k1zkp_v0_6_0_scalar_reduce(r, c + rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r));
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_mul_512(uint32_t *l, const rustsecp256k1zkp_v0_5_0_scalar *a, const rustsecp256k1zkp_v0_5_0_scalar *b) {
+static void rustsecp256k1zkp_v0_6_0_scalar_mul_512(uint32_t *l, const rustsecp256k1zkp_v0_6_0_scalar *a, const rustsecp256k1zkp_v0_6_0_scalar *b) {
     /* 96 bit accumulator. */
     uint32_t c0 = 0, c1 = 0, c2 = 0;
 
@@ -589,81 +569,20 @@ static void rustsecp256k1zkp_v0_5_0_scalar_mul_512(uint32_t *l, const rustsecp25
     l[15] = c0;
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_sqr_512(uint32_t *l, const rustsecp256k1zkp_v0_5_0_scalar *a) {
-    /* 96 bit accumulator. */
-    uint32_t c0 = 0, c1 = 0, c2 = 0;
-
-    /* l[0..15] = a[0..7]^2. */
-    muladd_fast(a->d[0], a->d[0]);
-    extract_fast(l[0]);
-    muladd2(a->d[0], a->d[1]);
-    extract(l[1]);
-    muladd2(a->d[0], a->d[2]);
-    muladd(a->d[1], a->d[1]);
-    extract(l[2]);
-    muladd2(a->d[0], a->d[3]);
-    muladd2(a->d[1], a->d[2]);
-    extract(l[3]);
-    muladd2(a->d[0], a->d[4]);
-    muladd2(a->d[1], a->d[3]);
-    muladd(a->d[2], a->d[2]);
-    extract(l[4]);
-    muladd2(a->d[0], a->d[5]);
-    muladd2(a->d[1], a->d[4]);
-    muladd2(a->d[2], a->d[3]);
-    extract(l[5]);
-    muladd2(a->d[0], a->d[6]);
-    muladd2(a->d[1], a->d[5]);
-    muladd2(a->d[2], a->d[4]);
-    muladd(a->d[3], a->d[3]);
-    extract(l[6]);
-    muladd2(a->d[0], a->d[7]);
-    muladd2(a->d[1], a->d[6]);
-    muladd2(a->d[2], a->d[5]);
-    muladd2(a->d[3], a->d[4]);
-    extract(l[7]);
-    muladd2(a->d[1], a->d[7]);
-    muladd2(a->d[2], a->d[6]);
-    muladd2(a->d[3], a->d[5]);
-    muladd(a->d[4], a->d[4]);
-    extract(l[8]);
-    muladd2(a->d[2], a->d[7]);
-    muladd2(a->d[3], a->d[6]);
-    muladd2(a->d[4], a->d[5]);
-    extract(l[9]);
-    muladd2(a->d[3], a->d[7]);
-    muladd2(a->d[4], a->d[6]);
-    muladd(a->d[5], a->d[5]);
-    extract(l[10]);
-    muladd2(a->d[4], a->d[7]);
-    muladd2(a->d[5], a->d[6]);
-    extract(l[11]);
-    muladd2(a->d[5], a->d[7]);
-    muladd(a->d[6], a->d[6]);
-    extract(l[12]);
-    muladd2(a->d[6], a->d[7]);
-    extract(l[13]);
-    muladd_fast(a->d[7], a->d[7]);
-    extract_fast(l[14]);
-    VERIFY_CHECK(c1 == 0);
-    l[15] = c0;
-}
-
 #undef sumadd
 #undef sumadd_fast
 #undef muladd
 #undef muladd_fast
-#undef muladd2
 #undef extract
 #undef extract_fast
 
-static void rustsecp256k1zkp_v0_5_0_scalar_mul(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a, const rustsecp256k1zkp_v0_5_0_scalar *b) {
+static void rustsecp256k1zkp_v0_6_0_scalar_mul(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *a, const rustsecp256k1zkp_v0_6_0_scalar *b) {
     uint32_t l[16];
-    rustsecp256k1zkp_v0_5_0_scalar_mul_512(l, a, b);
-    rustsecp256k1zkp_v0_5_0_scalar_reduce_512(r, l);
+    rustsecp256k1zkp_v0_6_0_scalar_mul_512(l, a, b);
+    rustsecp256k1zkp_v0_6_0_scalar_reduce_512(r, l);
 }
 
-static int rustsecp256k1zkp_v0_5_0_scalar_shr_int(rustsecp256k1zkp_v0_5_0_scalar *r, int n) {
+static int rustsecp256k1zkp_v0_6_0_scalar_shr_int(rustsecp256k1zkp_v0_6_0_scalar *r, int n) {
     int ret;
     VERIFY_CHECK(n > 0);
     VERIFY_CHECK(n < 16);
@@ -679,13 +598,7 @@ static int rustsecp256k1zkp_v0_5_0_scalar_shr_int(rustsecp256k1zkp_v0_5_0_scalar
     return ret;
 }
 
-static void rustsecp256k1zkp_v0_5_0_scalar_sqr(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a) {
-    uint32_t l[16];
-    rustsecp256k1zkp_v0_5_0_scalar_sqr_512(l, a);
-    rustsecp256k1zkp_v0_5_0_scalar_reduce_512(r, l);
-}
-
-static void rustsecp256k1zkp_v0_5_0_scalar_split_128(rustsecp256k1zkp_v0_5_0_scalar *r1, rustsecp256k1zkp_v0_5_0_scalar *r2, const rustsecp256k1zkp_v0_5_0_scalar *k) {
+static void rustsecp256k1zkp_v0_6_0_scalar_split_128(rustsecp256k1zkp_v0_6_0_scalar *r1, rustsecp256k1zkp_v0_6_0_scalar *r2, const rustsecp256k1zkp_v0_6_0_scalar *k) {
     r1->d[0] = k->d[0];
     r1->d[1] = k->d[1];
     r1->d[2] = k->d[2];
@@ -704,17 +617,17 @@ static void rustsecp256k1zkp_v0_5_0_scalar_split_128(rustsecp256k1zkp_v0_5_0_sca
     r2->d[7] = 0;
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_scalar_eq(const rustsecp256k1zkp_v0_5_0_scalar *a, const rustsecp256k1zkp_v0_5_0_scalar *b) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_eq(const rustsecp256k1zkp_v0_6_0_scalar *a, const rustsecp256k1zkp_v0_6_0_scalar *b) {
     return ((a->d[0] ^ b->d[0]) | (a->d[1] ^ b->d[1]) | (a->d[2] ^ b->d[2]) | (a->d[3] ^ b->d[3]) | (a->d[4] ^ b->d[4]) | (a->d[5] ^ b->d[5]) | (a->d[6] ^ b->d[6]) | (a->d[7] ^ b->d[7])) == 0;
 }
 
-SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_mul_shift_var(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a, const rustsecp256k1zkp_v0_5_0_scalar *b, unsigned int shift) {
+SECP256K1_INLINE static void rustsecp256k1zkp_v0_6_0_scalar_mul_shift_var(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *a, const rustsecp256k1zkp_v0_6_0_scalar *b, unsigned int shift) {
     uint32_t l[16];
     unsigned int shiftlimbs;
     unsigned int shiftlow;
     unsigned int shifthigh;
     VERIFY_CHECK(shift >= 256);
-    rustsecp256k1zkp_v0_5_0_scalar_mul_512(l, a, b);
+    rustsecp256k1zkp_v0_6_0_scalar_mul_512(l, a, b);
     shiftlimbs = shift >> 5;
     shiftlow = shift & 0x1F;
     shifthigh = 32 - shiftlow;
@@ -726,10 +639,10 @@ SECP256K1_INLINE static void rustsecp256k1zkp_v0_5_0_scalar_mul_shift_var(rustse
     r->d[5] = shift < 352 ? (l[5 + shiftlimbs] >> shiftlow | (shift < 320 && shiftlow ? (l[6 + shiftlimbs] << shifthigh) : 0)) : 0;
     r->d[6] = shift < 320 ? (l[6 + shiftlimbs] >> shiftlow | (shift < 288 && shiftlow ? (l[7 + shiftlimbs] << shifthigh) : 0)) : 0;
     r->d[7] = shift < 288 ? (l[7 + shiftlimbs] >> shiftlow)  : 0;
-    rustsecp256k1zkp_v0_5_0_scalar_cadd_bit(r, 0, (l[(shift - 1) >> 5] >> ((shift - 1) & 0x1f)) & 1);
+    rustsecp256k1zkp_v0_6_0_scalar_cadd_bit(r, 0, (l[(shift - 1) >> 5] >> ((shift - 1) & 0x1f)) & 1);
 }
 
-static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_scalar_cmov(rustsecp256k1zkp_v0_5_0_scalar *r, const rustsecp256k1zkp_v0_5_0_scalar *a, int flag) {
+static SECP256K1_INLINE void rustsecp256k1zkp_v0_6_0_scalar_cmov(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *a, int flag) {
     uint32_t mask0, mask1;
     VG_CHECK_VERIFY(r->d, sizeof(r->d));
     mask0 = flag + ~((uint32_t)0);
@@ -757,7 +670,7 @@ static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_scalar_cmov(rustsecp256k1zk
 #define LE32(p) (p)
 #endif
 
-static void rustsecp256k1zkp_v0_5_0_scalar_chacha20(rustsecp256k1zkp_v0_5_0_scalar *r1, rustsecp256k1zkp_v0_5_0_scalar *r2, const unsigned char *seed, uint64_t idx) {
+static void rustsecp256k1zkp_v0_6_0_scalar_chacha20(rustsecp256k1zkp_v0_6_0_scalar *r1, rustsecp256k1zkp_v0_6_0_scalar *r2, const unsigned char *seed, uint64_t idx) {
     size_t n;
     size_t over_count = 0;
     uint32_t seed32[8];
@@ -829,8 +742,8 @@ static void rustsecp256k1zkp_v0_5_0_scalar_chacha20(rustsecp256k1zkp_v0_5_0_scal
         r2->d[1] = x14;
         r2->d[0] = x15;
 
-        over1 = rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r1);
-        over2 = rustsecp256k1zkp_v0_5_0_scalar_check_overflow(r2);
+        over1 = rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r1);
+        over2 = rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r2);
         over_count++;
    } while (over1 | over2);
 }
@@ -839,4 +752,92 @@ static void rustsecp256k1zkp_v0_5_0_scalar_chacha20(rustsecp256k1zkp_v0_5_0_scal
 #undef QUARTERROUND
 #undef LE32
 
+static void rustsecp256k1zkp_v0_6_0_scalar_from_signed30(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_modinv32_signed30 *a) {
+    const uint32_t a0 = a->v[0], a1 = a->v[1], a2 = a->v[2], a3 = a->v[3], a4 = a->v[4],
+                   a5 = a->v[5], a6 = a->v[6], a7 = a->v[7], a8 = a->v[8];
+
+    /* The output from rustsecp256k1zkp_v0_6_0_modinv32{_var} should be normalized to range [0,modulus), and
+     * have limbs in [0,2^30). The modulus is < 2^256, so the top limb must be below 2^(256-30*8).
+     */
+    VERIFY_CHECK(a0 >> 30 == 0);
+    VERIFY_CHECK(a1 >> 30 == 0);
+    VERIFY_CHECK(a2 >> 30 == 0);
+    VERIFY_CHECK(a3 >> 30 == 0);
+    VERIFY_CHECK(a4 >> 30 == 0);
+    VERIFY_CHECK(a5 >> 30 == 0);
+    VERIFY_CHECK(a6 >> 30 == 0);
+    VERIFY_CHECK(a7 >> 30 == 0);
+    VERIFY_CHECK(a8 >> 16 == 0);
+
+    r->d[0] = a0       | a1 << 30;
+    r->d[1] = a1 >>  2 | a2 << 28;
+    r->d[2] = a2 >>  4 | a3 << 26;
+    r->d[3] = a3 >>  6 | a4 << 24;
+    r->d[4] = a4 >>  8 | a5 << 22;
+    r->d[5] = a5 >> 10 | a6 << 20;
+    r->d[6] = a6 >> 12 | a7 << 18;
+    r->d[7] = a7 >> 14 | a8 << 16;
+
+#ifdef VERIFY
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_scalar_check_overflow(r) == 0);
+#endif
+}
+
+static void rustsecp256k1zkp_v0_6_0_scalar_to_signed30(rustsecp256k1zkp_v0_6_0_modinv32_signed30 *r, const rustsecp256k1zkp_v0_6_0_scalar *a) {
+    const uint32_t M30 = UINT32_MAX >> 2;
+    const uint32_t a0 = a->d[0], a1 = a->d[1], a2 = a->d[2], a3 = a->d[3],
+                   a4 = a->d[4], a5 = a->d[5], a6 = a->d[6], a7 = a->d[7];
+
+#ifdef VERIFY
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_scalar_check_overflow(a) == 0);
+#endif
+
+    r->v[0] =  a0                   & M30;
+    r->v[1] = (a0 >> 30 | a1 <<  2) & M30;
+    r->v[2] = (a1 >> 28 | a2 <<  4) & M30;
+    r->v[3] = (a2 >> 26 | a3 <<  6) & M30;
+    r->v[4] = (a3 >> 24 | a4 <<  8) & M30;
+    r->v[5] = (a4 >> 22 | a5 << 10) & M30;
+    r->v[6] = (a5 >> 20 | a6 << 12) & M30;
+    r->v[7] = (a6 >> 18 | a7 << 14) & M30;
+    r->v[8] =  a7 >> 16;
+}
+
+static const rustsecp256k1zkp_v0_6_0_modinv32_modinfo rustsecp256k1zkp_v0_6_0_const_modinfo_scalar = {
+    {{0x10364141L, 0x3F497A33L, 0x348A03BBL, 0x2BB739ABL, -0x146L, 0, 0, 0, 65536}},
+    0x2A774EC1L
+};
+
+static void rustsecp256k1zkp_v0_6_0_scalar_inverse(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *x) {
+    rustsecp256k1zkp_v0_6_0_modinv32_signed30 s;
+#ifdef VERIFY
+    int zero_in = rustsecp256k1zkp_v0_6_0_scalar_is_zero(x);
+#endif
+    rustsecp256k1zkp_v0_6_0_scalar_to_signed30(&s, x);
+    rustsecp256k1zkp_v0_6_0_modinv32(&s, &rustsecp256k1zkp_v0_6_0_const_modinfo_scalar);
+    rustsecp256k1zkp_v0_6_0_scalar_from_signed30(r, &s);
+
+#ifdef VERIFY
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_scalar_is_zero(r) == zero_in);
+#endif
+}
+
+static void rustsecp256k1zkp_v0_6_0_scalar_inverse_var(rustsecp256k1zkp_v0_6_0_scalar *r, const rustsecp256k1zkp_v0_6_0_scalar *x) {
+    rustsecp256k1zkp_v0_6_0_modinv32_signed30 s;
+#ifdef VERIFY
+    int zero_in = rustsecp256k1zkp_v0_6_0_scalar_is_zero(x);
+#endif
+    rustsecp256k1zkp_v0_6_0_scalar_to_signed30(&s, x);
+    rustsecp256k1zkp_v0_6_0_modinv32_var(&s, &rustsecp256k1zkp_v0_6_0_const_modinfo_scalar);
+    rustsecp256k1zkp_v0_6_0_scalar_from_signed30(r, &s);
+
+#ifdef VERIFY
+    VERIFY_CHECK(rustsecp256k1zkp_v0_6_0_scalar_is_zero(r) == zero_in);
+#endif
+}
+
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_scalar_is_even(const rustsecp256k1zkp_v0_6_0_scalar *a) {
+    return !(a->d[0] & 1);
+}
+
 #endif /* SECP256K1_SCALAR_REPR_IMPL_H */
