@@ -19,12 +19,39 @@
 typedef struct {
     void (*fn)(const char *text, void* data);
     const void* data;
-} rustsecp256k1zkp_v0_5_0_callback;
+} rustsecp256k1zkp_v0_6_0_callback;
 
-static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_callback_call(const rustsecp256k1zkp_v0_5_0_callback * const cb, const char * const text) {
+static SECP256K1_INLINE void rustsecp256k1zkp_v0_6_0_callback_call(const rustsecp256k1zkp_v0_6_0_callback * const cb, const char * const text) {
     cb->fn(text, (void*)cb->data);
 }
 
+#ifndef USE_EXTERNAL_DEFAULT_CALLBACKS
+static void rustsecp256k1zkp_v0_6_0_default_illegal_callback_fn(const char* str, void* data) {
+    (void)data;
+    fprintf(stderr, "[libsecp256k1] illegal argument: %s\n", str);
+    abort();
+}
+static void rustsecp256k1zkp_v0_6_0_default_error_callback_fn(const char* str, void* data) {
+    (void)data;
+    fprintf(stderr, "[libsecp256k1] internal consistency check failed: %s\n", str);
+    abort();
+}
+#else
+void rustsecp256k1zkp_v0_6_0_default_illegal_callback_fn(const char* str, void* data);
+void rustsecp256k1zkp_v0_6_0_default_error_callback_fn(const char* str, void* data);
+#endif
+
+static const rustsecp256k1zkp_v0_6_0_callback default_illegal_callback = {
+    rustsecp256k1zkp_v0_6_0_default_illegal_callback_fn,
+    NULL
+};
+
+static const rustsecp256k1zkp_v0_6_0_callback default_error_callback = {
+    rustsecp256k1zkp_v0_6_0_default_error_callback_fn,
+    NULL
+};
+
+
 #ifdef DETERMINISTIC
 #define TEST_FAILURE(msg) do { \
     fprintf(stderr, "%s\n", msg); \
@@ -99,38 +126,8 @@ static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_callback_call(const rustsec
 
 #define ROUND_TO_ALIGN(size) ((((size) + ALIGNMENT - 1) / ALIGNMENT) * ALIGNMENT)
 
-/* Assume there is a contiguous memory object with bounds [base, base + max_size)
- * of which the memory range [base, *prealloc_ptr) is already allocated for usage,
- * where *prealloc_ptr is an aligned pointer. In that setting, this functions
- * reserves the subobject [*prealloc_ptr, *prealloc_ptr + alloc_size) of
- * alloc_size bytes by increasing *prealloc_ptr accordingly, taking into account
- * alignment requirements.
- *
- * The function returns an aligned pointer to the newly allocated subobject.
- *
- * This is useful for manual memory management: if we're simply given a block
- * [base, base + max_size), the caller can use this function to allocate memory
- * in this block and keep track of the current allocation state with *prealloc_ptr.
- *
- * It is VERIFY_CHECKed that there is enough space left in the memory object and
- * *prealloc_ptr is aligned relative to base.
- */
-static SECP256K1_INLINE void *manual_alloc(void** prealloc_ptr, size_t alloc_size, void* base, size_t max_size) {
-    size_t aligned_alloc_size = ROUND_TO_ALIGN(alloc_size);
-    void* ret;
-    VERIFY_CHECK(prealloc_ptr != NULL);
-    VERIFY_CHECK(*prealloc_ptr != NULL);
-    VERIFY_CHECK(base != NULL);
-    VERIFY_CHECK((unsigned char*)*prealloc_ptr >= (unsigned char*)base);
-    VERIFY_CHECK(((unsigned char*)*prealloc_ptr - (unsigned char*)base) % ALIGNMENT == 0);
-    VERIFY_CHECK((unsigned char*)*prealloc_ptr - (unsigned char*)base + aligned_alloc_size <= max_size);
-    ret = *prealloc_ptr;
-    *prealloc_ptr = (unsigned char*)*prealloc_ptr + aligned_alloc_size;
-    return ret;
-}
-
 /* Extract the sign of an int64, take the abs and return a uint64, constant time. */
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_sign_and_abs64(uint64_t *out, int64_t in) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_sign_and_abs64(uint64_t *out, int64_t in) {
     uint64_t mask0, mask1;
     int ret;
     ret = in < 0;
@@ -141,7 +138,7 @@ SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_sign_and_abs64(uint64_t *out
     return ret;
 }
 
-SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_clz64_var(uint64_t x) {
+SECP256K1_INLINE static int rustsecp256k1zkp_v0_6_0_clz64_var(uint64_t x) {
     int ret;
     if (!x) {
         return 64;
@@ -212,7 +209,7 @@ SECP256K1_INLINE static int rustsecp256k1zkp_v0_5_0_clz64_var(uint64_t x) {
 #endif
 
 /* Zero memory if flag == 1. Flag must be 0 or 1. Constant time. */
-static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_memczero(void *s, size_t len, int flag) {
+static SECP256K1_INLINE void rustsecp256k1zkp_v0_6_0_memczero(void *s, size_t len, int flag) {
     unsigned char *p = (unsigned char *)s;
     /* Access flag with a volatile-qualified lvalue.
        This prevents clang from figuring out (after inlining) that flag can
@@ -231,7 +228,7 @@ static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_memczero(void *s, size_t le
  * We use this to avoid possible compiler bugs with memcmp, e.g.
  * https://gcc.gnu.org/bugzilla/show_bug.cgi?id=95189
  */
-static SECP256K1_INLINE int rustsecp256k1zkp_v0_5_0_memcmp_var(const void *s1, const void *s2, size_t n) {
+static SECP256K1_INLINE int rustsecp256k1zkp_v0_6_0_memcmp_var(const void *s1, const void *s2, size_t n) {
     const unsigned char *p1 = s1, *p2 = s2;
     size_t i;
 
@@ -245,7 +242,7 @@ static SECP256K1_INLINE int rustsecp256k1zkp_v0_5_0_memcmp_var(const void *s1, c
 }
 
 /** If flag is true, set *r equal to *a; otherwise leave it. Constant-time.  Both *r and *a must be initialized and non-negative.*/
-static SECP256K1_INLINE void rustsecp256k1zkp_v0_5_0_int_cmov(int *r, const int *a, int flag) {
+static SECP256K1_INLINE void rustsecp256k1zkp_v0_6_0_int_cmov(int *r, const int *a, int flag) {
     unsigned int mask0, mask1, r_masked, a_masked;
     /* Access flag with a volatile-qualified lvalue.
        This prevents clang from figuring out (after inlining) that flag can
@@ -286,4 +283,69 @@ SECP256K1_GNUC_EXT typedef __int128 int128_t;
 # endif
 #endif
 
+#ifndef __has_builtin
+#define __has_builtin(x) 0
+#endif
+
+/* Determine the number of trailing zero bits in a (non-zero) 32-bit x.
+ * This function is only intended to be used as fallback for
+ * rustsecp256k1zkp_v0_6_0_ctz32_var, but permits it to be tested separately. */
+static SECP256K1_INLINE int rustsecp256k1zkp_v0_6_0_ctz32_var_debruijn(uint32_t x) {
+    static const uint8_t debruijn[32] = {
+        0x00, 0x01, 0x02, 0x18, 0x03, 0x13, 0x06, 0x19, 0x16, 0x04, 0x14, 0x0A,
+        0x10, 0x07, 0x0C, 0x1A, 0x1F, 0x17, 0x12, 0x05, 0x15, 0x09, 0x0F, 0x0B,
+        0x1E, 0x11, 0x08, 0x0E, 0x1D, 0x0D, 0x1C, 0x1B
+    };
+    return debruijn[((x & -x) * 0x04D7651F) >> 27];
+}
+
+/* Determine the number of trailing zero bits in a (non-zero) 64-bit x.
+ * This function is only intended to be used as fallback for
+ * rustsecp256k1zkp_v0_6_0_ctz64_var, but permits it to be tested separately. */
+static SECP256K1_INLINE int rustsecp256k1zkp_v0_6_0_ctz64_var_debruijn(uint64_t x) {
+    static const uint8_t debruijn[64] = {
+        0, 1, 2, 53, 3, 7, 54, 27, 4, 38, 41, 8, 34, 55, 48, 28,
+        62, 5, 39, 46, 44, 42, 22, 9, 24, 35, 59, 56, 49, 18, 29, 11,
+        63, 52, 6, 26, 37, 40, 33, 47, 61, 45, 43, 21, 23, 58, 17, 10,
+        51, 25, 36, 32, 60, 20, 57, 16, 50, 31, 19, 15, 30, 14, 13, 12
+    };
+    return debruijn[((x & -x) * 0x022FDD63CC95386D) >> 58];
+}
+
+/* Determine the number of trailing zero bits in a (non-zero) 32-bit x. */
+static SECP256K1_INLINE int rustsecp256k1zkp_v0_6_0_ctz32_var(uint32_t x) {
+    VERIFY_CHECK(x != 0);
+#if (__has_builtin(__builtin_ctz) || SECP256K1_GNUC_PREREQ(3,4))
+    /* If the unsigned type is sufficient to represent the largest uint32_t, consider __builtin_ctz. */
+    if (((unsigned)UINT32_MAX) == UINT32_MAX) {
+        return __builtin_ctz(x);
+    }
+#endif
+#if (__has_builtin(__builtin_ctzl) || SECP256K1_GNUC_PREREQ(3,4))
+    /* Otherwise consider __builtin_ctzl (the unsigned long type is always at least 32 bits). */
+    return __builtin_ctzl(x);
+#else
+    /* If no suitable CTZ builtin is available, use a (variable time) software emulation. */
+    return rustsecp256k1zkp_v0_6_0_ctz32_var_debruijn(x);
+#endif
+}
+
+/* Determine the number of trailing zero bits in a (non-zero) 64-bit x. */
+static SECP256K1_INLINE int rustsecp256k1zkp_v0_6_0_ctz64_var(uint64_t x) {
+    VERIFY_CHECK(x != 0);
+#if (__has_builtin(__builtin_ctzl) || SECP256K1_GNUC_PREREQ(3,4))
+    /* If the unsigned long type is sufficient to represent the largest uint64_t, consider __builtin_ctzl. */
+    if (((unsigned long)UINT64_MAX) == UINT64_MAX) {
+        return __builtin_ctzl(x);
+    }
+#endif
+#if (__has_builtin(__builtin_ctzll) || SECP256K1_GNUC_PREREQ(3,4))
+    /* Otherwise consider __builtin_ctzll (the unsigned long long type is always at least 64 bits). */
+    return __builtin_ctzll(x);
+#else
+    /* If no suitable CTZ builtin is available, use a (variable time) software emulation. */
+    return rustsecp256k1zkp_v0_6_0_ctz64_var_debruijn(x);
+#endif
+}
+
 #endif /* SECP256K1_UTIL_H */
