@@ -11,7 +11,7 @@ pub const WHITELIST_MAX_N_KEYS: size_t = 255;
 extern "C" {
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_pedersen_commitment_parse"
+        link_name = "rustsecp256k1zkp_v0_6_0_pedersen_commitment_parse"
     )]
     // Parse a 33-byte commitment into 64 byte internal commitment object
     pub fn secp256k1_pedersen_commitment_parse(
@@ -22,7 +22,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_pedersen_commitment_serialize"
+        link_name = "rustsecp256k1zkp_v0_6_0_pedersen_commitment_serialize"
     )]
     // Serialize a 64-byte commit object into a 33 byte serialized byte sequence
     pub fn secp256k1_pedersen_commitment_serialize(
@@ -33,7 +33,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_pedersen_commit"
+        link_name = "rustsecp256k1zkp_v0_6_0_pedersen_commit"
     )]
     // Generates a pedersen commitment: *commit = blind * G + value * G2.
     // The commitment is 33 bytes, the blinding factor is 32 bytes.
@@ -47,7 +47,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_pedersen_blind_generator_blind_sum"
+        link_name = "rustsecp256k1zkp_v0_6_0_pedersen_blind_generator_blind_sum"
     )]
     /// Sets the final Pedersen blinding factor correctly when the generators themselves
     ///  have blinding factors.
@@ -89,7 +89,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_pedersen_verify_tally"
+        link_name = "rustsecp256k1zkp_v0_6_0_pedersen_verify_tally"
     )]
     // Takes two list of 64-byte commitments and sums the first set and
     // subtracts the second and verifies that they sum to 0.
@@ -104,7 +104,7 @@ extern "C" {
     #[cfg(feature = "std")]
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_rangeproof_info"
+        link_name = "rustsecp256k1zkp_v0_6_0_rangeproof_info"
     )]
     pub fn secp256k1_rangeproof_info(
         ctx: *const Context,
@@ -119,7 +119,7 @@ extern "C" {
     #[cfg(feature = "std")]
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_rangeproof_rewind"
+        link_name = "rustsecp256k1zkp_v0_6_0_rangeproof_rewind"
     )]
     pub fn secp256k1_rangeproof_rewind(
         ctx: *const Context,
@@ -141,7 +141,7 @@ extern "C" {
     #[cfg(feature = "std")]
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_rangeproof_verify"
+        link_name = "rustsecp256k1zkp_v0_6_0_rangeproof_verify"
     )]
     pub fn secp256k1_rangeproof_verify(
         ctx: *const Context,
@@ -158,7 +158,7 @@ extern "C" {
     #[cfg(feature = "std")]
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_rangeproof_sign"
+        link_name = "rustsecp256k1zkp_v0_6_0_rangeproof_sign"
     )]
     pub fn secp256k1_rangeproof_sign(
         ctx: *const Context,
@@ -180,7 +180,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_surjectionproof_initialize"
+        link_name = "rustsecp256k1zkp_v0_6_0_surjectionproof_initialize"
     )]
     pub fn secp256k1_surjectionproof_initialize(
         ctx: *const Context,
@@ -196,7 +196,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_surjectionproof_serialize"
+        link_name = "rustsecp256k1zkp_v0_6_0_surjectionproof_serialize"
     )]
     pub fn secp256k1_surjectionproof_serialize(
         ctx: *const Context,
@@ -207,7 +207,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_surjectionproof_serialized_size"
+        link_name = "rustsecp256k1zkp_v0_6_0_surjectionproof_serialized_size"
     )]
     pub fn secp256k1_surjectionproof_serialized_size(
         ctx: *const Context,
@@ -216,7 +216,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_surjectionproof_parse"
+        link_name = "rustsecp256k1zkp_v0_6_0_surjectionproof_parse"
     )]
     pub fn secp256k1_surjectionproof_parse(
         ctx: *const Context,
@@ -227,7 +227,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_surjectionproof_generate"
+        link_name = "rustsecp256k1zkp_v0_6_0_surjectionproof_generate"
     )]
     pub fn secp256k1_surjectionproof_generate(
         ctx: *const Context,
@@ -242,7 +242,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_surjectionproof_verify"
+        link_name = "rustsecp256k1zkp_v0_6_0_surjectionproof_verify"
     )]
     pub fn secp256k1_surjectionproof_verify(
         ctx: *const Context,
@@ -254,7 +254,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_generator_generate_blinded"
+        link_name = "rustsecp256k1zkp_v0_6_0_generator_generate_blinded"
     )]
     pub fn secp256k1_generator_generate_blinded(
         ctx: *const Context,
@@ -265,7 +265,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_generator_serialize"
+        link_name = "rustsecp256k1zkp_v0_6_0_generator_serialize"
     )]
     pub fn secp256k1_generator_serialize(
         ctx: *const Context,
@@ -275,7 +275,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_generator_parse"
+        link_name = "rustsecp256k1zkp_v0_6_0_generator_parse"
     )]
     pub fn secp256k1_generator_parse(
         ctx: *const Context,
@@ -285,13 +285,13 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_nonce_function_ecdsa_adaptor"
+        link_name = "rustsecp256k1zkp_v0_6_0_nonce_function_ecdsa_adaptor"
     )]
     pub static secp256k1_nonce_function_ecdsa_adaptor: EcdsaAdaptorNonceFn;
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_ecdsa_adaptor_encrypt"
+        link_name = "rustsecp256k1zkp_v0_6_0_ecdsa_adaptor_encrypt"
     )]
     pub fn secp256k1_ecdsa_adaptor_encrypt(
         cx: *const Context,
@@ -305,7 +305,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_ecdsa_adaptor_verify"
+        link_name = "rustsecp256k1zkp_v0_6_0_ecdsa_adaptor_verify"
     )]
     pub fn secp256k1_ecdsa_adaptor_verify(
         cx: *const Context,
@@ -317,7 +317,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_ecdsa_adaptor_decrypt"
+        link_name = "rustsecp256k1zkp_v0_6_0_ecdsa_adaptor_decrypt"
     )]
     pub fn secp256k1_ecdsa_adaptor_decrypt(
         cx: *const Context,
@@ -328,7 +328,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_ecdsa_adaptor_recover"
+        link_name = "rustsecp256k1zkp_v0_6_0_ecdsa_adaptor_recover"
     )]
     pub fn secp256k1_ecdsa_adaptor_recover(
         cx: *const Context,
@@ -340,7 +340,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_whitelist_signature_parse"
+        link_name = "rustsecp256k1zkp_v0_6_0_whitelist_signature_parse"
     )]
     pub fn secp256k1_whitelist_signature_parse(
         cx: *const Context,
@@ -351,7 +351,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_whitelist_signature_serialize"
+        link_name = "rustsecp256k1zkp_v0_6_0_whitelist_signature_serialize"
     )]
     pub fn secp256k1_whitelist_signature_serialize(
         ctx: *const Context,
@@ -362,7 +362,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_whitelist_sign"
+        link_name = "rustsecp256k1zkp_v0_6_0_whitelist_sign"
     )]
     pub fn secp256k1_whitelist_sign(
         ctx: *const Context,
@@ -380,7 +380,7 @@ extern "C" {
 
     #[cfg_attr(
         not(feature = "external-symbols"),
-        link_name = "rustsecp256k1zkp_v0_5_0_whitelist_verify"
+        link_name = "rustsecp256k1zkp_v0_6_0_whitelist_verify"
     )]
     pub fn secp256k1_whitelist_verify(
         ctx: *const Context,
@@ -490,12 +490,6 @@ impl Default for Tag {
     }
 }
 
-impl hash::Hash for Tag {
-    fn hash<H: hash::Hasher>(&self, state: &mut H) {
-        self.0.hash(state)
-    }
-}
-
 impl From<[u8; 32]> for Tag {
     fn from(bytes: [u8; 32]) -> Self {
         Tag(bytes)
@@ -526,12 +520,6 @@ impl Default for PedersenCommitment {
     }
 }
 
-impl hash::Hash for PedersenCommitment {
-    fn hash<H: hash::Hasher>(&self, state: &mut H) {
-        state.write(&self.0)
-    }
-}
-
 /// A ring signature for the "whitelist" scheme.
 #[repr(C)]
 #[derive(Clone)]
